# -*- coding: utf-8 -*-
#
# Copyright (c) 2016, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

from chipwhisperer.hardware.naeusb.fpga import FPGA
from chipwhisperer.hardware.naeusb.naeusb import NAEUSB
from chipwhisperer.hardware.naeusb.programmer_avr import AVRISP
from chipwhisperer.hardware.naeusb.programmer_xmega import XMEGAPDI
from chipwhisperer.hardware.naeusb.programmer_stm32fserial import STM32FSerial
from chipwhisperer.hardware.naeusb.serial import USART


class CWLiteUSB(object):
    _name = "ChipWisperer-Lite USB"

    def __init__(self):
        self._cwusb = NAEUSB()

        # Connect required modules up here
        self.fpga = FPGA(self._cwusb)
        self.xmega = XMEGAPDI(self._cwusb)
        self.avr = AVRISP(self._cwusb)
        self.usart = USART(self._cwusb)
        self.serialstm32f = STM32FSerial(cwserial=self.usart, cwapi=None)

    def get_possible_devices(self, idProduct):
        return self._cwusb.get_possible_devices(idProduct=idProduct)

    def con(self, *args, **kwargs):
        return self._cwusb.con(*args, **kwargs)

    def dis(self):
        pass

    def usbdev(self):
        return self._cwusb




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2015, NewAE Technology Inc
# All rights reserved.
#
# Authors: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging
import zipfile
import io
import base64
from collections import OrderedDict
from chipwhisperer.capture.scopes.cwhardware import PartialReconfiguration as pr
from chipwhisperer.common.utils import util

glitchaddr = 51
glitchoffsetaddr = 25
glitchreadbackaddr = 56
CODE_READ       = 0x80
CODE_WRITE      = 0xC0

# sign extend b low bits in x
# from "Bit Twiddling Hacks"
def SIGNEXT(x, b):
    m = 1 << (b - 1)
    x = x & ((1 << b) - 1)
    return (x ^ m) - m

class GlitchSettings(util.DisableNewAttr):

    # Output modes, sorted by ID (FPGA value)
    _output_modes = [
        "clock_xor",
        "clock_or",
        "glitch_only",
        "clock_only",
        "enable_only"
    ]

    # Trigger types
    _glitch_triggers = [
        "manual",
        "ext_continuous",
        "continuous",
        "ext_single"
    ]

    def __init__(self, cwglitch):
        self.cwg = cwglitch
        self.disable_newattr()

    def _dict_repr(self):
        dict = OrderedDict()

        dict['clk_src'] = self.clk_src

        dict['width'] = self.width
        dict['width_fine'] = self.width_fine
        dict['offset'] = self.offset
        dict['offset_fine'] = self.offset_fine
        dict['trigger_src'] = self.trigger_src
        dict['arm_timing'] = self.arm_timing
        dict['ext_offset'] = self.ext_offset
        dict['repeat'] = self.repeat
        dict['output'] = self.output

        return dict

    def __repr__(self):
        return util.dict_to_str(self._dict_repr())

    def __str__(self):
        return self.__repr__()

    def manualTrigger(self):
        """Manually trigger the glitch output.

        This trigger is most useful in Manual trigger mode, where this is the
        only way to cause a glitch.
        """
        self.cwg.glitchManual()

    def readStatus(self):
        """Read the status of the two glitch DCMs.

        Returns:
            A tuple with 4 elements::

             * phase1: Phase shift of DCM1,
             * phase2: Phase shift of DCM2,
             * lock1: Whether DCM1 is locked,
             * lock2: Whether DCM2 is locked
        """
        return self.cwg.getDCMStatus()

    def resetDcms(self):
        """Reset the two glitch DCMs.

        This is automatically run after changing the glitch width or offset,
        so this function is typically not necessary.
        """
        self.cwg.resetDCMs()

    @property
    def clk_src(self):
        """The clock signal that the glitch DCM is using as input.

        This DCM can be clocked from two different sources:
         * "target": The HS1 clock from the target device
         * "clkgen": The CLKGEN DCM output

        :Getter:
           Return the clock signal currently in use

        :Setter:
           Change the glitch clock source

        Raises:
           ValueError: New value not one of "target" or "clkgen"
        """
        clk_val = self.cwg.glitchClkSource()
        if clk_val == self.cwg.CLKSOURCE0_BIT:
            return "target"
        elif clk_val == self.cwg.CLKSOURCE1_BIT:
            return "clkgen"
        else:
            raise ValueError("Received unexpected glitch module clock source %s" % (clk_val), clk_val)

    @clk_src.setter
    def clk_src(self, source):
        if source == "target":
            clk_val = self.cwg.CLKSOURCE0_BIT
        elif source == "clkgen":
            clk_val = self.cwg.CLKSOURCE1_BIT
        else:
            raise ValueError("Can't set glitch arm timing to %s; valid values: ('target', 'clkgen')" % source, source)
        self.cwg.setGlitchClkSource(clk_val)

    @property
    def width(self):
        """The width of a single glitch pulse, as a percentage of one period.

        One pulse can range from -49.8% to roughly 49.8% of a period. The
        system may not be reliable at 0%. Note that negative widths are allowed;
        these act as if they are positive widths on the other half of the
        clock cycle.

        :Getter: Return a float with the current glitch width.

        :Setter: Update the glitch pulse width. The value will be adjusted to
            the closest possible glitch width.

        Raises:
           UserWarning: Width outside of [-49.8, 49.8]. The value is rounded
               to one of these
        """
        return self.cwg.getGlitchWidth()

    @width.setter
    def width(self, value):
        if value < self.cwg._min_width or value > self.cwg._max_width:
            raise UserWarning("Can't use glitch width %s - rounding into [%s, %s]" % (value, self.cwg._min_width, self.cwg._max_width))
        self.cwg.setGlitchWidth(value)

    @property
    def width_fine(self):
        """The fine adjustment value on the glitch width.

        This is a dimensionless number that makes small adjustments to the
        glitch pulses' width. Valid range is [-255, 255].

        :Getter: Return the current glitch fine width

        :Setter: Update the glitch fine width

        Raises:
           TypeError: offset not an integer
           ValueError: offset is outside of [-255, 255]
        """
        return self.cwg.getGlitchWidthFine()

    @width_fine.setter
    def width_fine(self, value):
        try:
            int_val = int(value)
        except ValueError:
            raise TypeError("Can't convert %s to integer" % value, value)

        if int_val < -255 or int_val > 255:
            raise ValueError("New fine width is outside range [-255, 255]")
        self.cwg.setGlitchWidthFine(int_val)

    @property
    def offset(self):
        """The offset from a rising clock edge to a glitch pulse rising edge,
        as a percentage of one period.

        A pulse may begin anywhere from -49.8% to 49.8% away from a rising
        edge, allowing glitches to be swept over the entire clock cycle.

        :Getter: Return a float with the current glitch offset.

        :Setter: Set the glitch offset. The new value is rounded to the nearest
            possible offset.

        Raises:
           TypeError: offset not an integer
           UserWarning: value outside range [-50, 50] (value is rounded)
        """
        return self.cwg.getGlitchOffset()

    @offset.setter
    def offset(self, value):
        if value < self.cwg._min_offset or value > self.cwg._max_offset:
            raise UserWarning("Can't use glitch offset %s - rounding into [%s, %s]" % (value, self.cwg._min_offset, self.cwg._max_offset))
        self.cwg.setGlitchOffset(value)

    @property
    def offset_fine(self):
        """The fine adjustment value on the glitch offset.

        This is a dimensionless number that makes small adjustments to the
        glitch pulses' offset. Valid range is [-255, 255].

        :Getter: Return the current glitch fine offset

        :Setter: Update the glitch fine offset

        Raises:
           TypeError: if offset not an integer
           ValueError: if offset is outside of [-255, 255]
        """
        return self.cwg.getGlitchOffsetFine()

    @offset_fine.setter
    def offset_fine(self, value):
        try:
            int_val = int(value)
        except ValueError:
            raise TypeError("Can't convert %s to integer" % value, value)

        if int_val < -255 or int_val > 255:
            raise ValueError("New fine offset is outside range [-255, 255]")
        self.cwg.setGlitchOffsetFine(int_val)

    @property
    def trigger_src(self):
        """The trigger signal for the glitch pulses.

        The glitch module can use four different types of triggers:
         * "continuous": Constantly trigger glitches
         * "manual": Only trigger glitches through API calls/GUI actions
         * "ext_single": Use the trigger module. One glitch per scope arm.
         * "ext_continuous": Use the trigger module. Many glitches per arm.

        :Getter: Return the current trigger source.

        :Setter: Change the trigger source.

        Raises:
           ValueError: value not listed above.
        """
        trig_src = self.cwg.glitchTrigger()
        return self._glitch_triggers[trig_src]

    @trigger_src.setter
    def trigger_src(self, src):
        try:
            trig_idx = self._glitch_triggers.index(src)
        except ValueError:
            raise ValueError("Can't set glitch trigger to %s; valid values: %s" % (src, self._glitch_triggers), src)

        self.cwg.setGlitchTrigger(trig_idx)

    @property
    def arm_timing(self):
        """When to arm the glitch in single-shot mode.

        If the glitch module is in "ext_single" trigger mode, it must be armed
        when the scope is armed. There are two timings for this event:

         * "before_scope": The glitch module is armed first.
         * "after_scope": The scope is armed first. This is the default.

        This setting may be helpful if trigger events are happening very early.

        If the glitch module is not in external trigger single-shot mode, this
        setting has no effect.

        :Getter: Return the current arm timing ("before_scope" or "after_scope")

        :Setter: Change the arm timing

        Raises:
           ValueError: if value not listed above
        """
        timing = self.cwg.getArmTiming()
        if timing == 1:
            return "before_scope"
        elif timing == 2:
            return "after_scope"
        else:
            raise ValueError("Received unexpected glitch arm timing %s" % (timing), timing)

    @arm_timing.setter
    def arm_timing(self, value):
        if value == "before_scope":
            int_val = 1
        elif value == "after_scope":
            int_val = 2
        else:
            raise ValueError("Can't set glitch arm timing to %s; valid values: ('before_scope', 'after_scope')" % value, value)

        self.cwg.setArmTiming(int_val)

    @property
    def ext_offset(self):
        """How long the glitch module waits between a trigger and a glitch.

        After the glitch module is triggered, it waits for a number of clock
        cycles before generating glitch pulses. This delay allows the glitch to
        be inserted at a precise moment during the target's execution to glitch
        specific instructions.

        .. note::
            It is possible to get more precise offsets by clocking the
            glitch module faster than the target board.

        This offset must be in the range [0, 2**32).

        :Getter: Return the current external trigger offset.

        :Setter: Set the external trigger offset.

        Raises:
           TypeError: if offset not an integer
           ValueError: if offset outside of range [0, 2**32)
        """
        return self.cwg.triggerOffset()

    @ext_offset.setter
    def ext_offset(self, offset):
        try:
            int_val = int(offset)
        except ValueError:
            raise TypeError("Can't convert %s to integer" % offset, offset)

        if int_val < 0 or int_val >= 2**32:
            raise ValueError("New trigger offset %d is outside range [0, 2**32)" % int_val)

        self.cwg.setTriggerOffset(offset)

    @property
    def repeat(self):
        """The number of glitch pulses to generate per trigger.

        When the glitch module is triggered, it produces a number of pulses
        that can be combined with the clock signal. This setting allows for
        the glitch module to produce stronger glitches (especially during
        voltage glitching).

        Repeat counter must be in the range [1, 255].

        :Getter: Return the current repeat value (integer)

        :Setter: Set the repeat counter

        Raises:
           TypeError: if value not an integer
           ValueError: if value outside [1, 255]
        """
        return self.cwg.numGlitches()

    @repeat.setter
    def repeat(self, value):
        try:
            int_val = int(value)
        except ValueError:
            raise TypeError("Can't convert %s to integer" % value, value)

        if int_val < 1 or int_val > 255:
            raise ValueError("New repeat value %d is outside range [1, 255]", int_val)

        self.cwg.setNumGlitches(int_val)

    @property
    def output(self):
        """The type of output produced by the glitch module.

        There are 5 ways that the glitch module can combine the clock with its
        glitch pulses:

         * "clock_only": Output only the original input clock.
         * "glitch_only": Output only the glitch pulses - do not use the clock.
         * "clock_or": Output is high if either the clock or glitch are high.
         * "clock_xor": Output is high if clock and glitch are different.
         * "enable_only": Output is high for glitch.repeat cycles.

        Some of these settings are only useful in certain scenarios:
         * Clock glitching: "clock_or" or "clock_xor"
         * Voltage glitching: "glitch_only" or "enable_only"

        :Getter: Return the current glitch output mode (one of above strings)

        :Setter: Change the glitch output mode.

        Raises:
           ValueError: if value not in above strings
        """
        output_mode = self.cwg.glitchType()
        return self._output_modes[output_mode]

    @output.setter
    def output(self, value):
        try:
            output_idx = self._output_modes.index(value)
        except ValueError:
            raise ValueError("Can't set glitch mode to %s; valid values: %s" % (value, self._output_modes), value)
        self.cwg.setGlitchType(output_idx)

class ChipWhispererGlitch(object):
    """
    Drives the Glitch Module inside the ChipWhisperer Capture Hardware Rev2, or can be used to drive this FPGA
     module inserted into other systems.
    """
    CLKSOURCE0_BIT = 0b00000000
    CLKSOURCE1_BIT = 0b00000001
    CLKSOURCE_MASK = 0b00000011
    _name= 'Glitch Module'

    def __init__(self, cwtype, scope, oa):

        # Setup FPGA partial configuration dataZ
        self.prCon = pr.PartialReconfigConnection()
        self.prEnabled = False
        self.oa = oa

        # Glitch width/offset
        # Note that these are ints scaled by 256/100
        self._width = 26
        self._offset = 26

        # These ranges are updated during __init__: see below
        self._min_width = 0
        self._max_width = 100

        self._min_offset = 0
        self._max_offset = 100

        # Single-shot arm timing
        self._ssarm = 2

        # Check if we've got partial reconfiguration stuff for this scope
        try:
            if cwtype == "cwrev2" or cwtype == "cwcrev2":
                settingprefix = "cwcrev2"
                partialbasename = "s6lx25"
                self.glitchPR = pr.PartialReconfigDataMulti()
            elif cwtype == "cwlite":
                settingprefix = "cwlite"
                partialbasename = "cwlite"
                self.glitchPR = pr.PartialReconfigDataMulti()
            elif cwtype == "cw1200":
                settingprefix = "cw1200"
                partialbasename = "cw1200"
                self.glitchPR = pr.PartialReconfigDataMulti()
            else:
                raise ValueError("Invalid ChipWhisperer Mode: %s" % cwtype)

            if scope.getFWConfig().loader._release_mode != "debug":

                if scope.getFWConfig().loader._release_mode == "builtin":
                    filelike = scope.getFWConfig().loader._bsBuiltinData
                    zfile = zipfile.ZipFile(filelike)
                elif scope.getFWConfig().loader._release_mode == "zipfile":
                    fileloc = scope.getFWConfig().loader._bsZipLoc
                    if zipfile.is_zipfile(fileloc):
                        zfile = zipfile.ZipFile(fileloc, "r")
                    else:
                        logging.warning('Partial Reconfiguration DISABLED: no zip-file for FPGA')
                        zfile = None
                else:
                    logging.warning('Partial Reconfiguration DISABLED: no PR data for FPGA')
                    zfile = None
                    raise ValueError("Unknown FPGA mode: %s"%scope.getFWConfig().loader._release_mode)

                if zfile:
                    self.glitchPR.load(zfile.open("%s-glitchwidth.p" % partialbasename))
                    self.glitchPR.load(zfile.open("%s-glitchoffset.p" % partialbasename))
                    self.prEnabled = True
                else:
                    self.prEnabled = False
            else:
                logging.warning('Partial Reconfiguration DISABLED: Debug bitstream mode')
                self.prEnabled = False

        except IOError as e:
            logging.error(str(e))
            self.prEnabled = False
        except ValueError as e:
            logging.error(str(e))
            self.prEnabled = False
        except OSError as e:  # Also catches WindowsError
            logging.error(str(e))
            self.prEnabled = False

        if self.prEnabled:
            # Enable glitch width, check what we've got access to
            self._min_width = self.glitchPR.limitList[0][0] / 2.55
            self._max_width = self.glitchPR.limitList[0][1] / 2.55
            self._min_offset = self.glitchPR.limitList[1][0] / 2.55
            self._max_offset = self.glitchPR.limitList[1][1] / 2.55

        self.setOpenADC(oa)
        self.glitchSettings = GlitchSettings(self)

    def setOpenADC(self, oa):
        self.oa = None
        if self.prEnabled:
            self.prCon.con(oa)

            # Check this is actually working
            if self.prCon.isPresent() == False:
                self.prEnabled = False
                logging.warning('Partial Reconfiguration block not detected, PR disabled')
                return

            # Reset FPGA back to defaults in case previous bitstreams loaded
            self.updatePartialReconfig()
        self.oa = oa

    def updatePartialReconfig(self, _=None):
        """
        Reads the values set via the GUI & updates the hardware settings for partial reconfiguration. Checks that PR
        is enabled with self.prEnabled.
        """

        if not self.prEnabled:
            return

        if self._width == 0:
            logging.warning('Partial reconfiguration for width = 0 may not work')

        if self._offset == 0:
            logging.warning('Partial reconfiguration for offset = 0 may not work')

        bs = self.glitchPR.getPartialBitstream([self._width, self._offset])

        if self.prEnabled:
            self.prCon.program(bs)
            if self.oa is not None:
                self.resetDCMs(keepPhase=True)
                # print "Partial: %d %d" % (widthint, offsetint)

            self.updateGlitchReadBack()

    def updateGlitchReadBack(self, test=False):
        """Updates the readback register in the FPGA with glitch information, used for LCD update on CW1200 hardware."""

        #Don't write if PR disable by accident
        if self.oa is None:
            return

        width = self.getGlitchWidth()
        offset = self.getGlitchOffset()

        cmd = bytearray(8)

        #Integer downloads
        cmd[0] = self._offset & 0xff
        cmd[1] = (self._offset >> 8) & 0xff
        cmd[2] = self._width & 0xff
        cmd[3] = (self._width >> 8) & 0xff

        #Floating point info
        cmd[4] = int(offset) & 0xff
        cmd[5] = int(("%f"%offset).split(".")[1][0:2]) & 0xff

        cmd[6] = int(width) & 0xff
        cmd[7] = int(("%f"%width).split(".")[1][0:2]) & 0xff

        self.oa.sendMessage(CODE_WRITE, glitchreadbackaddr, cmd, Validate=False)

    def setGlitchWidth(self, width):
        if width > self._max_width:
            width = self._max_width
        if width < self._min_width:
            width = self._min_width
        self._width = int(round((width/100.) * 256.))

        self.updatePartialReconfig()

    def getGlitchWidth(self):
        return self._width * 100. / 256.

    def setGlitchOffset(self, offset):
        if offset > self._max_offset:
            offset = self._max_offset
        if offset < self._min_offset:
            offset = self._min_offset
        self._offset = int(round((offset / 100.) * 256.))

        self.updatePartialReconfig()

    def getGlitchOffset(self):
        return self._offset * 100. / 256.

    def setTriggerOffset(self, offset):
        offset = int(offset)
        """Set offset between trigger event and glitch in clock cycles"""
        cmd = bytearray(4)
        cmd[0] = ((offset >> 0) & 0xFF)
        cmd[1] = ((offset >> 8) & 0xFF)
        cmd[2] = ((offset >> 16) & 0xFF)
        cmd[3] = ((offset >> 24) & 0xFF)
        self.oa.sendMessage(CODE_WRITE, glitchoffsetaddr, cmd)

    def triggerOffset(self):
        """Get offset between trigger event and glitch in clock cycles"""
        cmd = self.oa.sendMessage(CODE_READ, glitchoffsetaddr, maxResp=4)
        offset = cmd[0]
        offset |= cmd[1] << 8
        offset |= cmd[2] << 16
        offset |= cmd[3] << 24
        return offset

    def setGlitchOffsetFine(self, fine):
        """Set the fine glitch offset adjust, range -255 to 255"""
        current = self.oa.sendMessage(CODE_READ, glitchaddr, Validate=False, maxResp=8)

        if current is None or len(current) < 8:
            logging.warning('Glitch Module not present?')
            return

        LSB = fine & 0x00FF
        MSB = (fine & 0x0100) >> 8

        current[0] = LSB #7..0
        current[1] = (current[1] & ~0x01) | MSB #15..8

        #Start adjust
        current[2] = current[2] | 0x04  # 23..16
        #assign clockglitch_settings_read[37] = phase1_done_reg;
        #assign clockglitch_settings_read[38] = phase2_done_reg;

        self.oa.sendMessage(CODE_WRITE, glitchaddr, current, Validate=False)

    def getGlitchWidthFine(self):
        return self.getDCMStatus()[1]

    def setGlitchWidthFine(self, fine):
        """Set the fine glitch width adjust, range -255 to 255"""
        current = self.oa.sendMessage(CODE_READ, glitchaddr, Validate=False, maxResp=8)

        if current is None or len(current) < 8:
            logging.warning('Glitch Module not present?')
            return

        LSB = fine & 0x00FF
        MSB = (fine & 0x0100) >> 8

        current[1] = (current[1] & 0x01) | ((LSB & 0x7F) << 1)
        current[2] = (current[2] & ~0x03) | ((LSB >> 7) | (MSB << 1))

        #Start adjust
        current[2] = current[2] | 0x04  # 23..16
        #assign clockglitch_settings_read[37] = phase1_done_reg;
        # assign clockglitch_settings_read[38] = phase2_done_reg;
        self.oa.sendMessage(CODE_WRITE, glitchaddr, current, Validate=False)

    def getGlitchOffsetFine(self):
        return self.getDCMStatus()[0]

    def getDCMStatus(self):
        current = self.oa.sendMessage(CODE_READ, glitchaddr, Validate=False, maxResp=8)

        glitch_offset_fine_loaded = current[2] >> 3
        glitch_offset_fine_loaded |= (current[3] & 0x0F) << 5
        glitch_offset_fine_loaded = SIGNEXT(glitch_offset_fine_loaded, 9)

        glitch_width_fine_loaded = (current[3] & 0xF0) >> 4
        glitch_width_fine_loaded |= (current[4] & 0x1F) << 4
        glitch_width_fine_loaded = SIGNEXT(glitch_width_fine_loaded, 9)

        dcm1Lock = False
        dcm2Lock = False

        if current[4] & 0x80:
            dcm1Lock = True

        if current[5] & 0x01:
            dcm2Lock = True

        return (glitch_offset_fine_loaded, glitch_width_fine_loaded, dcm1Lock, dcm2Lock)

    def actionResetDCMs(self, _=None):
        """Action for parameter class
            ..todo:: See if this method is still needed of if it's GUI only
        """
        self.resetDCMs()

    def resetDCMs(self, keepPhase=False):
        """Reset the DCMs for the Glitch width & Glitch offset. Required after doing a PR operation"""

        reset = self.oa.sendMessage(CODE_READ, glitchaddr, Validate=False, maxResp=8)
        reset[5] |= (1<<1)
        self.oa.sendMessage(CODE_WRITE, glitchaddr, reset, Validate=False)
        reset[5] &= ~(1<<1)
        self.oa.sendMessage(CODE_WRITE, glitchaddr, reset, Validate=False)

    def checkLocked(self, _=None):
        """Check if the DCMs are locked and print results """

        stat = self.getDCMStatus()
        logging.info('DCM1: Phase %d, Locked %r' % (stat[0], stat[2]))
        logging.info('DCM2: Phase %d, Locked %r' % (stat[1], stat[3]))

    def setNumGlitches(self, num):
        """Set number of glitches to occur after a trigger"""
        num = int(num)
        resp = self.oa.sendMessage(CODE_READ, glitchaddr, Validate=False, maxResp=8)

        if resp is None or len(resp) < 8:
            logging.warning('Glitch Module not present?')
            return

        if num < 1:
            num = 1
        resp[6] = num-1
        self.oa.sendMessage(CODE_WRITE, glitchaddr, resp, Validate=False)

    def numGlitches(self):
        """Get number of glitches to occur after a trigger"""
        resp = self.oa.sendMessage(CODE_READ, glitchaddr, Validate=False, maxResp=8)
        return resp[6]+1

    def setGlitchTrigger(self, trigger):
        """Set glitch trigger type (manual, continous, adc-trigger)"""
        resp = self.oa.sendMessage(CODE_READ, glitchaddr, Validate=False, maxResp=8)
        resp[5] = (resp[5] & ~(0x0C)) | (trigger << 2)
        self.oa.sendMessage(CODE_WRITE, glitchaddr, resp, Validate=False)

    def glitchTrigger(self):
        """Get glitch trigger type"""
        resp = self.oa.sendMessage(CODE_READ, glitchaddr, Validate=False, maxResp=8)
        return (resp[5] & 0x0C) >> 2

    def setGlitchType(self, t):
        """Set glitch output type (ORd with clock, XORd with clock, clock only, glitch only)"""
        resp = self.oa.sendMessage(CODE_READ, glitchaddr, Validate=False, maxResp=8)
        resp[5] = (resp[5] & ~(0x70)) | (t << 4)
        self.oa.sendMessage(CODE_WRITE, glitchaddr, resp, Validate=False)

    def glitchType(self):
        resp = self.oa.sendMessage(CODE_READ, glitchaddr, Validate=False, maxResp=8)
        return (resp[5] & 0x70) >> 4

    def glitchManual(self, _=None):
        """
        Cause a single glitch event to occur. Depending on setting of numGlitches() this may mean
        multiple glitches in a row
        """
        resp = self.oa.sendMessage(CODE_READ, glitchaddr, Validate=False, maxResp=8)
        resp[5] = resp[5] | (1 << 7)
        self.oa.sendMessage(CODE_WRITE, glitchaddr, resp, Validate=False)
        resp[5] = resp[5] & ~(1 << 7)
        self.oa.sendMessage(CODE_WRITE, glitchaddr, resp, Validate=False)

    def glitchArm(self):
        """If trigger is set to single-shot mode, this must be called before the selected trigger occurs"""
        self.glitchManual()

    def setGlitchClkSource(self, source):
        """Set the source of the glitched clock, either the HS1-In or the CLKGEN Module"""
        resp = self.oa.sendMessage(CODE_READ, glitchaddr, Validate=False, maxResp=8)
        resp[7] = (resp[7] & ~self.CLKSOURCE_MASK) | source
        self.oa.sendMessage(CODE_WRITE, glitchaddr, resp, Validate=False)

    def glitchClkSource(self):
        resp = self.oa.sendMessage(CODE_READ, glitchaddr, Validate=False, maxResp=8)
        return (resp[7] & self.CLKSOURCE_MASK)

    def getArmTiming(self):
        return self._ssarm

    def setArmTiming(self, val):
        self._ssarm = val

    def armPreScope(self):
        """Called before scope trigger is armed"""
        if self.getArmTiming() == 1:
            self.glitchArm()

    def armPostScope(self):
        """Called after scope trigger is armed"""
        if self.getArmTiming() == 2:
            self.glitchArm()



#
# This script packs firmware files into Python modules to allow easy distribution
#
import os
import base64
import datetime
import binascii

hwdir = r"../../../../hardware/"

#List of versions, file-names, and object name
cw305_v = [0, 11]
cw305_files = []

cwcr2_v = [0, 11]
cwcr2_files = [("cwrev2_firmware.zip",  os.path.join(hwdir, r"capture/chipwhisperer-rev2/cwrev2_firmware.zip")),
               ("OpenADC.ihx", os.path.join(hwdir, r"capture/chipwhisperer-rev2/ezusb-firmware/ztex-sdk/examples/usb-fpga-1.11/1.11c/openadc/OpenADC.ihx"))]

cwlite_v = [0, 11]
cwlite_files = [("cwlite_firmware.zip", os.path.join(hwdir, r"capture/chipwhisperer-lite/cwlite_firmware.zip")),
                ("SAM3U_CW1173.bin", os.path.join(hwdir, r"capture/chipwhisperer-lite/sam3u_fw/SAM3U_VendorExample/Debug/SAM3U_CW1173.bin"))]

cw1200_v = [1, 0]
cw1200_files = [("cw1200_firmware.zip", os.path.join(hwdir, r"capture/chipwhisperer-cw1200/cw1200_firmware.zip")),
                ]

#List of files to generate
file_list = [
#    ("cw305.py" , cw305_v , cw305_files),
#    ("cwcr2.py" , cwcr2_v , cwcr2_files),
#    ("cwlite.py", cwlite_v, cwlite_files),
    ("cw1200.py", cw1200_v, cw1200_files),
]

for fdata in file_list:
    f = open(fdata[0], "w")

    f.write("# This file was auto-generated. Do not manually edit or save. What are you doing looking at it? Close it now!\n")
    f.write("# Generated on %s\n"%datetime.datetime.now())
    f.write("#\n")
    f.write("import binascii\n")
    f.write("import io\n\n")
    f.write("fwver = [%d, %d]\n" % (fdata[1][0], fdata[1][1]))
    f.write("def getsome(item, filelike=True):\n")
    f.write("    data = _contents[item].encode('latin-1')\n")
    f.write("    data = binascii.a2b_base64(data)\n")
    f.write("    if filelike:\n")
    f.write("        data = io.BytesIO(data)\n")
    f.write("    return data\n\n")
    f.write("_contents = {\n")

    f.write("")

    for embdata in fdata[2]:
        with open(embdata[1], "rb") as e_file:
            # json_str = base64.b64encode(e_file.read())# json.dumps(e_file.read(), ensure_ascii=False)
            json_str = binascii.b2a_base64(e_file.read())

            f.write("\n#Contents from %s\n"%embdata[1])
            f.write("'%s':'"%embdata[0])
            f.write(json_str.decode().replace("\n",""))
            f.write("',\n\n")
            f.flush()
    f.write("}\n")

f.close()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2016, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging

from ._base import ReaderTemplate
from chipwhisperer.capture.scopes.cwhardware import ChipWhispererTargets


class ReaderChipWhispererUSI(ReaderTemplate):
    _name = "CWCR2-USI (obsolete)"

    def __init__(self):
        ReaderTemplate.__init__(self)
        self.getParams().addChildren([
            {'name':'Baud', 'type':'int', 'value':9600, 'action':lambda p: self.setBaud(p.getValue())}
        ])
        self.usi = ChipWhispererTargets.CWUniversalSerial()

    def setBaud(self, brate):
        """Set baud rate of reader"""

        #For SmartCard we assume Stop-bits = 1, guard-bits=1, start-bits=1, parity=even
        self.usi.setBaud(brate)

    def sendAPDU(self, cla, ins, p1, p2, txdata=None, rxdatalen=0):
        """Send APDU to SmartCard, get Response"""

        #SmartCard header
        data = [cla,ins, p1, p2]

        if txdata is not None:
            txdatalen = len(txdata)
            data.append(txdatalen)
        else:
            txdatalen = 0

        if (txdata is None) & (rxdatalen == 0):
            data.append(0)

        #Append gap during which SCard sends ACK
        data.append(None)

        #Append payload
        if txdata is not None:
            for b in txdata: data.append(b)

        if rxdatalen != 0:
            data.append(rxdatalen)

        #Append requested response size

        #Additional processing time
        proctime = 8

        #Start RX to catch response
        self.usi.read(2+rxdatalen+txdatalen+proctime, startonly=True)
        self.usi.write(data)
        temprx = bytearray(self.usi.read(2 + rxdatalen + txdatalen + proctime, waitonly=True))

        # Uncomment to print data stream on RX pin
        stream = 'APDU: '
        for t in temprx:
            stream += "%02x " % t
        logging.info(stream)

        # We've actually read entire sent + received data in
        # Strip header
        del temprx[0:5]

        # Strip transmitted data
        if txdatalen > 0:
            del temprx[1:(txdatalen + 1)]

        # Strip receive data length, present only if TX + RX happened
        if rxdatalen > 0 and txdatalen > 0:
            del temprx[1]

        p = temprx

        if p[0] != ins:
            print(("ACK Error: %x != %x"%(ins, p[0])))

        #for t in p:
        #    print "%x "%t,

        if len(p) < 2:
            raise IOError("Payload too small: %d, %s" % (len(p), str(p)))

        status = (p[-2] << 8) | p[-1]

        if rxdatalen > 0:
            payload = p[1:-2]
            return (status, payload)

        return status

    def con(self, scope=None):
        """Connect to reader. scope parameter is OpenADC/ChipWhisperer hardware, only used to integrated readers"""
        self.usi.con(scope.qtadc.ser)
        self.usi.setIdle(1)
        self.setBaud(9600)
        self.usi.setParity("even")
        self.usi.setStopbits(2)

    def flush(self):
        """Discard all input buffers"""
        pass

    def reset(self):
        """Reset card & save the ATR"""
        pass

    def getATR(self):
        """Get the ATR from the SmartCard. Reads a saved value, user reset() to actually reset card."""
        pass



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
__author__ = "Colin O'Flynn"

from . import TraceContainerNative
try:
    from . import TraceContainerMySQL
except ImportError:
    TraceContainerMySQL = None

from . import TraceContainerDPAv3

TraceContainerFormatList = {"native":TraceContainerNative.TraceContainerNative, "dpav3":TraceContainerDPAv3.TraceContainerDPAv3 }
if TraceContainerMySQL is not None:
    TraceContainerFormatList["mysql"] = TraceContainerMySQL.TraceContainerMySQL



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2017, NewAE Technology Inc
# All rights reserved.
#
# Authors: Colin O'Flynn, Greg d'Eon
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

# This file is a modified version of @slaypni's FastDTW Python library, which is licensed under the MIT license:
# The MIT License (MIT)
#
# Copyright (c) 2015 Kazuaki Tanida
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, division

import numbers
import numpy as np
from collections import defaultdict

try:
    range = xrange
except NameError:
    pass


def fastdtw(x, y, radius=1, dist=None):
    ''' return the approximate distance between 2 time series with O(N)
        time and memory complexity

        Parameters
        ----------
        x : array_like
            input array 1
        y : array_like
            input array 2
        radius : int
            size of neighborhood when expanding the path. A higher value will
            increase the accuracy of the calculation but also increase time
            and memory consumption. A radius equal to the size of x and y will
            yield an exact dynamic time warping calculation.
        dist : function or int
            The method for calculating the distance between x[i] and y[j]. If
            dist is an int of value p > 0, then the p-norm will be used. If
            dist is a function then dist(x[i], y[j]) will be used. If dist is
            None then abs(x[i] - y[j]) will be used.

        Returns
        -------
        float
            the approximate distance between the 2 time series

        Examples
        --------
        >>> import numpy as np
        >>> import fastdtw
        >>> x = np.array([1, 2, 3, 4, 5], dtype='float')
        >>> y = np.array([2, 3, 4], dtype='float')
        >>> fastdtw.fastdtw(x, y)
        (2.0, [(0, 0), (1, 0), (2, 1), (3, 2), (4, 2)])
    '''

    if not isinstance(x, np.ndarray):
        x = np.array(x)
    if not isinstance(y, np.ndarray):
        y = np.array(y)
    if x.ndim == y.ndim > 1 and x.shape[1] != y.shape[1]:
        raise ValueError('second dimension of x and y must be the same')
    if isinstance(dist, numbers.Number) and dist <= 0:
        raise ValueError('dist cannot be a negative integer')

    if dist is None:
        dist = __difference
    elif isinstance(dist, numbers.Number):
        dist = __norm(p=dist)

    return __fastdtw(x, y, radius, dist)


def __difference(a, b):
    return abs(a - b)


def __norm(p):
    return lambda a, b: np.linalg.norm(a - b, p)


def __fastdtw(x, y, radius, dist):
    min_time_size = radius + 2

    if len(x) < min_time_size or len(y) < min_time_size:
        return dtw(x, y, dist=dist)

    x_shrinked = __reduce_by_half(x)
    y_shrinked = __reduce_by_half(y)
    distance, path = \
        __fastdtw(x_shrinked, y_shrinked, radius=radius, dist=dist)
    window = __expand_window(path, len(x), len(y), radius)

    ret = dtw(x, y, window, dist=dist)
    return ret


def dtw(x, y, window=None, dist=lambda a, b: abs(a - b)):
    ''' return the distance between 2 time series without approximation

        Parameters
        ----------
        x : array_like
            input array 1
        y : array_like
            input array 2
        dist : function or int
            The method for calculating the distance between x[i] and y[j]. If
            dist is an int of value p > 0, then the p-norm will be used. If
            dist is a function then dist(x[i], y[j]) will be used. If dist is
            None then abs(x[i] - y[j]) will be used.

        Returns
        -------
        float
            the approximate distance between the 2 time series

        Examples
        --------
        >>> import numpy as np
        >>> import fastdtw
        >>> x = np.array([1, 2, 3, 4, 5], dtype='float')
        >>> y = np.array([2, 3, 4], dtype='float')
        >>> fastdtw.dtw(x, y)
        (2.0, [(0, 0), (1, 0), (2, 1), (3, 2), (4, 2)])
    '''
    len_x, len_y = len(x), len(y)
    if window is None:
        window = [(i, j) for i in range(len_x) for j in range(len_y)]
    window = ((i + 1, j + 1) for i, j in window)
    D = defaultdict(lambda: (float('inf'),))
    D[0, 0] = (0, 0, 0)

    window = list(window)
    window_len = len(window)
    print(window_len)
    idx = 0
    if window_len > 10000:
        pass
    else:
        progress_bar = None
    for i,j in window:
        dt = dist(x[i-1], y[j-1])
        D[i, j] = (D[i-1, j-1][0], i-1, j-1)
        if D[i, j-1][0] < D[i, j][0]:
            D[i, j] = (D[i, j-1][0], i, j-1)
        if D[i-1, j][0] < D[i, j][0]:
            D[i, j] = (D[i-1, j][0], i-1, j)
        D[i, j] = (D[i, j][0] + dt, D[i,j][1], D[i,j][2])

        idx += 1
        if progress_bar is not None and idx % 1000 == 0:
            progress_bar.updateStatus(idx, idx)
            if progress_bar.wasAborted():
                raise RuntimeWarning("User aborted DTW operation")

    path = []
    i, j = len_x, len_y
    while not (i == j == 0):
        path.append((i-1, j-1))
        i, j = D[i, j][1], D[i, j][2]
    path.reverse()
    return (D[len_x, len_y][0], path)


def __reduce_by_half(x):
    return [(x[i] + x[1+i]) / 2 for i in range(0, len(x) - len(x) % 2, 2)]


def __expand_window(path, len_x, len_y, radius):
    path_ = set(path)
    for i, j in path:
        for a, b in ((i + a, j + b)
                     for a in range(-radius, radius+1)
                     for b in range(-radius, radius+1)):
            path_.add((a, b))

    window_ = set()
    for i, j in path_:
        for a, b in ((i * 2, j * 2), (i * 2, j * 2 + 1),
                     (i * 2 + 1, j * 2), (i * 2 + 1, j * 2 + 1)):
            window_.add((a, b))

    window = []
    start_j = 0
    for i in range(0, len_x):
        new_start_j = None
        for j in range(start_j, len_y):
            if (i, j) in window_:
                window.append((i, j))
                if new_start_j is None:
                    new_start_j = j
            elif new_start_j is not None:
                break
        start_j = new_start_j

    return window


#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Author: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

import random
import numpy as np
from ._base import PreprocessingBase
from chipwhisperer.common.utils.parameter import setupSetParam


class AddNoiseJitter(PreprocessingBase):
    """Add random time jitter to each of the traces.
    """
    _name = "Add Noise: Time Jitter"
    _description = "Add random jitter. This module is used for testing resyncronization modules, and has no use " \
                  "in actual analysis."

    def __init__(self, traceSource=None, name=None):
        PreprocessingBase.__init__(self, traceSource, name=name)
        self._maxJitter = 0
        self.getParams().addChildren([
            {'name':'Max Jitter (+/- cycles)', 'key':'jitter', 'type':'int', 'set':self._setJitter, 'get':self._getJitter, 'limits':(0, 1000)}
        ])

    def _getJitter(self):
        return self._maxJitter

    @setupSetParam("Max Jitter (+/- cycles)")
    def _setJitter(self, jit):
        """Change the maximum amount of jitter in cycles.

        Equivalent to
        >>> ppmod.jitter = jit
        """
        self._maxJitter = jit

    @property
    def jitter(self):
        """The maximum amount of jitter added to each trace, in cycles

        Setter raises TypeError if jitter isn't integer
        """
        return self._maxJitter

    @jitter.setter
    def jitter(self, jit):
        if not isinstance(jit, int):
            raise TypeError("Expected int; got %s" % type(jit), jit)
        self._setJitter(jit)
   
    def getTrace(self, n):
        if self.enabled:
            trace = self._traceSource.getTrace(n)
            if trace is None:
                return None
            
            jit = random.randint(-self._maxJitter, self._maxJitter)

            return roll_zeropad(trace, jit)
        else:
            return self._traceSource.getTrace(n)

        
# This function stolen from: http://stackoverflow.com/questions/2777907/python-numpy-roll-with-padding
def roll_zeropad(a, shift, axis=None):
    """
    Roll array elements along a given axis. Elements rolled off the end of the array are treated as zeros.

    :param a: Input Array
    :type a: array_like
    
    :param shift: The number of places by which elements are shifted.
    :type shift: int

    :param axis: The axis along which elements are shifted.  By default, the array
                 is flattened before shifting, after which the original
                 shape is restored.
    :type axis: int, optional      

    :rtype: ndarray, with the same shape as ``a``

    """
    a = np.asanyarray(a)
    if shift == 0: return a
    if axis is None:
        n = a.size
        reshape = True
    else:
        n = a.shape[axis]
        reshape = False
    if np.abs(shift) > n:
        res = np.zeros_like(a)
    elif shift < 0:
        shift += n
        zeros = np.zeros_like(a.take(np.arange(n - shift), axis))
        res = np.concatenate((a.take(np.arange(n - shift, n), axis), zeros), axis)
    else:
        zeros = np.zeros_like(a.take(np.arange(n - shift, n), axis))
        res = np.concatenate((zeros, a.take(np.arange(n - shift), axis)), axis)
    if reshape:
        return res.reshape(a.shape)
    else:
        return res



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.

import configparser
import logging
import os.path
import re

from chipwhisperer.common.traces.TraceContainerNative import TraceContainerNative
from chipwhisperer.common.utils import util
from chipwhisperer.common.utils.tracesource import TraceSource

from datetime import datetime
from pathlib import Path
import copy

class TraceManager(TraceSource):
    """
    When using traces in ChipWhisperer, you may have remapped a bunch of trace
    files into one segment of traces. This class is used to handle the
    remapping and provide methods to save, load and manage the traces.

    Don't use anything that modifies the trace manager -> stick to stuff
    that gives you information about it instead (i.e. get_trace, get_known_key)
    """

    def __init__(self, name = "Trace Management"):
        TraceSource.__init__(self, name)
        self.name = name
        self.dirty = util.Observable(False)
        self._numTraces = 0
        self._numPoints = 0
        self._sampleRate = 0
        self.lastUsedSegment = None
        self.traceSegments = []
        self.saved = False
        if __debug__: logging.debug('Created: ' + str(self))

    def new_project(self):
        """Create a new empty set of traces."""
        self.traceSegments = []
        self.dirty.setValue(False)
        self.sigTracesChanged.emit()

    newProject = new_project

    def save_project(self, config, configfilename):
        """Save the trace segments information to a project file."""
        config[self.name].clear()
        for indx, t in enumerate(self.traceSegments):
            starttime = datetime.now()
            prefix = starttime.strftime('%Y.%m.%d-%H.%M.%S') + "_"
            t.config.setConfigFilename(os.path.splitext(configfilename)[0] + "_data" + "/traces/config_" + prefix + ".cfg")
            t.config.setAttr("prefix", prefix)
            t.config.setAttr("date", starttime.strftime('%Y-%m-%d %H:%M:%S'))
            config[self.name]['tracefile%d' % indx] = os.path.normpath(os.path.relpath(t.config.configFilename(), os.path.split(configfilename)[0]))
            config[self.name]['enabled%d' % indx] = str(t.enabled)

            t.saveAllTraces(os.path.dirname(t.config.configFilename()), prefix)
        self.dirty.setValue(False)
        self.saved = True

    saveProject = util.camel_case_deprecated(save_project)

    def loadProject(self, configfilename):
        """Load the trace segments information from a project file."""
        config = configparser.RawConfigParser()
        config.read(configfilename)
        alltraces = config.items(self.name)
        self.newProject()

        path, name = os.path.split(configfilename)

        for t in alltraces:
            if t[0].startswith("tracefile"):
                fname = os.path.join(path, t[1])
                fname = os.path.normpath(fname.replace("\\", "/"))
                # print "Opening %s"%fname
                ti = TraceContainerNative()
                try:
                    ti.config.loadTrace(fname)
                    ti.loadAllTraces()
                except Exception as e:
                    logging.error(str(e))
                self.traceSegments.append(ti)
            if t[0].startswith("enabled"):
                tnum = re.findall(r'[0-9]+', t[0])
                self.traceSegments[int(tnum[0])].enabled = t[1] == "True"
        self._setModified()
        self.dirty.setValue(False)

    def removeTraceSegments(self, positions):
        """Remove a list of trace segments. Do not repeat numbers!!"""
        if not isinstance(positions, list):
            positions = [positions]
        else:
            positions.sort(reverse=True)
        for pos in positions:
            self.traceSegments.pop(pos)
        self._setModified()

    def setTraceSegmentStatus(self, pos, newStatus):
        """Set a trace segment as enabled/disabled."""
        self.traceSegments[pos].enabled = newStatus
        self._setModified()

    def getSegmentList(self, start=0, end=-1):
        """Return a list of segments."""
        tnum = start
        if end == -1:
            end = self._numTraces

        dataDict = {'offsetList':[], 'lengthList':[]}

        while(tnum < end):
            t = self.get_segment(tnum)
            dataDict['offsetList'].append(t.mappedRange[0])
            dataDict['lengthList'].append(t.mappedRange[1] - t.mappedRange[0] + 1)
            tnum = t.mappedRange[1] + 1

        return dataDict

    def get_segment(self, traceIndex):
        """Return the trace segment with the specified trace in the list with all enabled segments."""
        if self.lastUsedSegment is not None:
            if self.lastUsedSegment.mappedRange is not None and self.lastUsedSegment.mappedRange[0] <= traceIndex <= \
                    self.lastUsedSegment.mappedRange[1]:
                return self.lastUsedSegment
            else:
                # Only load one segment at a time for memory reasons
                # If the traces are actually saved :)
                if self.saved:
                    self.lastUsedSegment.unloadAllTraces()
                    self.lastUsedSegment = None

        for traceSegment in self.traceSegments:
            if traceSegment.mappedRange and traceSegment.mappedRange[0] <= traceIndex <= traceSegment.mappedRange[1]:
                if not traceSegment.isLoaded():
                    traceSegment.loadAllTraces(None, None)
                self.lastUsedSegment = traceSegment
                return traceSegment

        raise ValueError("Error: Trace %d is not in mapped range." % traceIndex)

    getSegment = util.camel_case_deprecated(get_segment)

    def getAuxData(self, n, auxDic):
        """Return data about a segment"""
        t = self.get_segment(n)
        cfg = t.getAuxDataConfig(auxDic)
        if cfg is not None:
            filedata = t.loadAuxData(cfg["filename"])
        else:
            filedata = None

        return {'cfgdata':cfg, 'filedata':filedata}

    def get_trace(self, n):
        """Return the trace with index n in the list of enabled segments"""
        t = self.get_segment(n)
        return t.getTrace(n - t.mappedRange[0])

    getTrace = util.camel_case_deprecated(get_trace)

    def get_textin(self, n):
        """Return the input text of trace with index n in the list of enabled segments"""
        t = self.get_segment(n)
        return t.getTextin(n - t.mappedRange[0])

    getTextin = util.camel_case_deprecated(get_textin)

    def get_textout(self, n):
        """Return the output text of trace with index n in the list of enabled segments"""
        t = self.get_segment(n)
        return t.getTextout(n - t.mappedRange[0])

    getTextout = util.camel_case_deprecated(get_textout)

    def get_known_key(self, n):
        """Return the known encryption key."""
        try:
            t = self.get_segment(n)
            return t.getKnownKey(n - t.mappedRange[0])
        except ValueError:
            return []

    getKnownKey = util.camel_case_deprecated(get_known_key)

    def _updateRanges(self):
        """Update the trace range for each segments."""
        startTrace = 0
        self._sampleRate = 0
        self._numPoints = 0
        for t in self.traceSegments:
            if t.enabled:
                tlen = t.numTraces()
                t.mappedRange = [startTrace, startTrace+tlen-1]
                startTrace = startTrace + tlen
                np = int(t.config.attr("numPoints"))
                if self._numPoints != np and np != 0:
                    if self._numPoints == 0:
                        self._numPoints = np
                    else:
                        logging.warning("Selected trace segments have different number of points: %d!=%d" % (self._numPoints, np))
                        self._numPoints = min(self._numPoints, np)

                sr = int(float(t.config.attr("scopeSampleRate")))
                if self._sampleRate != sr and sr != 0:
                    if self._sampleRate == 0:
                        self._sampleRate = sr
                    else:
                        logging.warning("Selected trace segments have different sample rates: %d!=%d" % (self._sampleRate, sr))
            else:
                t.mappedRange = None
        self._numTraces = startTrace

    def num_points(self):
        """Return the number of points in traces of the selected segments."""
        return self._numPoints

    numPoints = util.camel_case_deprecated(num_points)

    def num_traces(self):
        """Return the number of traces in the current list of enabled segments."""
        return self._numTraces

    numTraces = util.camel_case_deprecated(num_traces)

    def appendSegment(self, ti, enabled=True):
        """Append a new segment to the list of trace segments."""
        ti.enabled = enabled
        ti._isloaded = True
        self.traceSegments.append(ti)
        self._setModified()

    def _setModified(self):
        """Notify passive and active observers to be updated."""
        self.dirty.setValue(True)
        self._updateRanges()
        self.sigTracesChanged.emit()

    def getSampleRate(self, ):
        return self._sampleRate

    def changeSegmentAttribute(self, segmentNum, attribute, value):
        """Change the value of a segment attribute. Changes are saved instantly."""
        self.traceSegments[segmentNum].config.setAttr(attribute, value)
        self.traceSegments[segmentNum].config.saveTrace()
        logging.info('Trace attribute "%s" of segment %d changed to: %s' % (attribute, segmentNum, value))
        self._updateRanges()
        self.sigTracesChanged.emit()

    def __del__(self):
        if __debug__: logging.debug('Deleted: ' + str(self))



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Author: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

import numpy as np
from ._base import PreprocessingBase
from chipwhisperer.common.utils.parameter import setupSetParam


class AddNoiseRandom(PreprocessingBase):
    """Add Gaussian white noise to each of the traces.
    """
    _name = "Add Noise: Amplitude"
    _description = "Add random noise."

    def __init__(self, traceSource=None, name=None):
        PreprocessingBase.__init__(self, traceSource, name=name)
        self._noise_std_dev = 0.005
        self.params.addChildren([
            {'name':'Noise Std-Dev', 'key':'noisestddev', 'type':'float', 'step':0.001, 'limits':(0, 1.0), 'get':self._getNoise, 'set':self._setNoise}
        ])

    @setupSetParam("Noise Std-Dev")
    def _setNoise(self, std_dev):
        self._noise_std_dev = std_dev

    def _getNoise(self):
        return self._noise_std_dev

    @property
    def noise(self):
        """The standard deviation of the noise.

        If 0, the traces are passed through without being modified. Otherwise,
        random values from N(0, noise) are added to each of the points.

        Setter raises TypeError if noise level isn't a number
        """
        return self._getNoise()

    @noise.setter
    def noise(self, std_dev):
        if not isinstance(std_dev, (int, float)):
            raise TypeError("Expected number; got %s" % type(std_dev), std_dev)
        self._setNoise(std_dev)

    def getTrace(self, n):
        if self.enabled:
            trace = self._traceSource.getTrace(n)
            if trace is None:
                return None

            if self._noise_std_dev == 0:
                return trace
            else:
                return trace + np.random.normal(scale=self._noise_std_dev, size=len(trace))
        else:
            return self._traceSource.getTrace(n)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2017, NewAE Technology Inc
# All rights reserved.
#
# Authors: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.

from collections import OrderedDict
import inspect

from .base import ModelsBase

class XORLeakageHelper(object):

    name = 'XOR Leakage Model (unnamed)'

    def leakage(self, pt, ct, key, bnum):
        """
        Override this function with specific leakage function (S-Box output, HD, etc).

        Args:
            pt: 16-byte plain-text input
            ct: 16-byte cipher-text output.
            key: 16-byte AES key - byte 'bnum' may be a GUESS if key is known. Rest of bytes may/may not be valid too.
            bnum: Byte number we are trying to attack.

        Returns:
            Value that will be presented on the 8-bit bus. Leakage model (such as HW) will map this to leakage itself.
        """
        raise NotImplementedError("ASKLeakageHelper does not implement leakage")

class PtKey_XOR(XORLeakageHelper):
    name = 'HW: Key XOR Plaintext'
    def leakage(self, pt, ct, key, bnum):
        return pt[bnum] ^ key[bnum]


#List of all classes you can use
enc_list = [PtKey_XOR]
dec_list = []

class XOR_8bit(ModelsBase):
    _name = 'XOR'

    hwModels = OrderedDict((mod.name, mod) for mod in (enc_list+dec_list) )

    def __init__(self, model=PtKey_XOR, bitmask=0xFF):
        ModelsBase.__init__(self, 4, 256, model=model)

        #Override number of subkeys - todo make this work
        #self.numSubKeys = 2
        #self.getParams().addChildren([{'name': 'Number of SubKeys', 'type':'list', 'values':[1,2,4,8,16,32], 'get':self.getNumSubKeys, 'set':self.setNumSubKeys}])
        self._mask = bitmask


    def _updateHwModel(self):
        """" Re-implement this to update leakage model """
        self.modelobj = None

        #Check if they passed an object...
        if isinstance(self.model, XORLeakageHelper):
            self.modelobj = self.model

        #Check if they passed a class...
        elif inspect.isclass(self.model) and issubclass(self.model, XORLeakageHelper):
            self.modelobj = self.model()

        #Otherwise it's probably one of these older keys (kept for backwards-compatability)
        else:
            raise AttributeError("Unknown model")

        if self.modelobj is None:
            raise ValueError("Invalid model: %s" % str(self.model))

    def processKnownKey(self, inpkey):
        return inpkey

    def leakage(self, pt, ct, guess, bnum, state):
        try:
            #Make a copy so we don't screw with anything...
            key = list(state['knownkey'])
        except:
            #We don't log due to time-sensitive nature... but if state doesn't have "knownkey" will result in
            #unknown knownkey which causes some attacks to fail. Possibly should make this some sort of
            #flag to indicate we want to ignore the problem?
            key = [None]*16

        #Guess can be 'none' if we want to use original key as-is
        if guess is not None:
            key[bnum] = guess

        #Get intermediate value
        intermediate_value = self.modelobj.leakage(pt, ct, key, bnum)

        #For bit-wise attacks, mask off specific bit value
        intermediate_value = self._mask & intermediate_value

        #Return HW of guess
        return self.HW[intermediate_value]



#  Java host software API of ZTEX EZ-USB FX2 SDK
#  Python version based on code that is:
#  Copyright (C) 2009-2011 ZTEX GmbH.
#  http://www.ztex.de
#
#  Python conversion done via java2python + Colin O'Flynn's fixes
#  this version is VERY limited compared to the full Java version.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 3 as
#  published by the Free Software Foundation.
#  This program is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#  General Public License for more details.
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see http://www.gnu.org/licenses/.
import logging

import usb
import time

cypressVendorId = 0x4b4

#  * EZ-USB USB product ID: 0x8613
cypressProductId = 0x8613

#  * ZTEX vendor ID: 0x221a
ztexVendorId = 0x221A

#
#      * USB product ID for ZTEX devices that support ZTEX descriptor 1: 0x100.
#      * This product ID is intended for general purpose use and can be shared by all devices that base on ZTEX modules.
#      * Different products are identified by a second product ID, namely the PRODUCT_ID field of the <a href="#descriptor"> ZTEX descriptor 1</a>.
#      * <p>
#      * Please read the <a href="http://www.ztex.de/firmware-kit/usb_ids.e.html">informations about USB vendor and product ID's<a>.
#      * @see #ztexProductIdMax
#
ztexProductId = 0x100


class EzUsb(object):
    """ generated source for class EzUsb """
    #  ******* reset **************************************************************
    #
    #      * Controls the reset state of a Cypress EZ-USB device.
    #      * @param handle The handle of the device.
    #      * @param r The reset state (true means reset).
    #      * @throws FirmwareUploadException if an error occurred while attempting to control the reset state.
    #

    def __init__(self, dev):
        self.dev = dev

    def reset(self, r):
        """ generated source for method reset """
        buffer_ = [int((1 if r else 0))]

        k = self.dev.ctrl_transfer(0x40, 0xA0, 0xE600, 0, buffer_, 1000)

        #  upload j bytes
        if k < 0:
            raise IOError("unable to set reset=" + buffer_[0])
        elif k != 1:
            raise IOError("Unable to set reset=" + buffer_[0])

        # Give some recovery time
        time.sleep(0.050 if r else 0.400)

    #  ******* uploadFirmware ******************************************************
    #
    #      * Uploads the Firmware to a Cypress EZ-USB device.
    #      * @param handle The handle of the device.
    #      * @param ihxFile The firmware image.
    #      * @return the upload time in ms.
    #      * @throws FirmwareUploadException if an error occurred while attempting to upload the firmware.
    #
    def uploadFirmware(self, ihxFile):
        """ generated source for method uploadFirmware """
        transactionBytes = 256
        buffer_ = [None] * transactionBytes
        self.reset(True)

        #  reset = 1
        # t0 = Date().getTime()
        j = 0
        i = 0
        for i in range(0, len(ihxFile.ihxData) + 1):
            if i >= len(ihxFile.ihxData) or ihxFile.ihxData[i] < 0 or j >= transactionBytes:
                if j > 0:
                    #  upload j bytes
                    k = self.dev.ctrl_transfer(0x40, 0xA0, i - j, 0, buffer_[0:j], 1000)
                    if k < 0:
                        raise IOError("Unknown USB Error #1?")
                    elif k != j:
                        raise IOError("Unknown USB Error #2?")

                    time.sleep(0.001)
                j = 0

            if i < len(ihxFile.ihxData) and ihxFile.ihxData[i] >= 0 and ihxFile.ihxData[i] <= 255:
                buffer_[j] = int(ihxFile.ihxData[i])
                j += 1
            i += 1
        # t1 = Date().getTime()
        try:
            self.reset(False)
            #  error (may caused re-numeration) can be ignored
        except IOError:
            pass
        # return t1 - t0
        return 0


class IhxFile(object):
    """ generated source for class IhxFile """
    #
    #      * This array stores the firmware image.
    #      * Values &lt;0 and &gt;255 mean that the data is undefined.
    #
    ihxData = [None] * 65536

    #  ******* readHexDigit ********************************************************
    @classmethod
    def readHexDigit(self, in_):
        """ generated source for method readHexDigit """
        b = in_.read(1)
        if len(b) == 0:
            raise ValueError("Inexpected end of file")
        b = ord(b)
        if b >= ord('0') and b <= ord('9'):
            return b - ord('0')
        if b >= ord('a') and b <= ord('f'):
            return 10 + b - ord('a')
        if b >= ord('A') and b <= ord('F'):
            return 10 + b - ord('A')
        raise ValueError("Hex digit expected: " + chr(b))

    #  ******* readHexByte *********************************************************
    def readHexByte(self, in_):
        """ generated source for method readHexByte """
        data = (self.readHexDigit(in_) << 4) | self.readHexDigit(in_)
        # print "%x" % data
        return data

    #  ******* IhxFile *************************************************************
    #
    #      * Constructs an instance from a given file name.
    #      * This method can also read system resources, e.g. files from the current jar archive.
    #      * @param fileName The file name.
    #      * @throws IOException If an read error occurred.
    #      * @throws IhxFileDamagedException If the ihx file is damaged.
    #
    def __init__(self, fname):
        """ generated source for method __init__ """

        #Allow usage of both filelike type and filenames
        if hasattr(fname, "read"):
            in_ = fname
        else:
            in_ = open(fname, "r")
        b = None
        lenfield = int()
        cs = int()
        addr = int()
        buf = [None] * 255
        eof = False
        line = 0

        for i in range(0, len(self.ihxData)):
            self.ihxData[i] = -1

        try:
            while not eof:
                while b != ':':
                    b = in_.read(1)
                    if len(b) == 0: raise ValueError("Unexpected end of file")

                line += 1
                lenfield = self.readHexByte(in_)
                #  length field
                cs = lenfield
                b = self.readHexByte(in_)
                #  address field
                cs += b
                addr = b << 8
                b = self.readHexByte(in_)
                cs += b
                addr |= b
                b = self.readHexByte(in_)
                #  record type field
                cs += b
                for i in range(0, lenfield):
                    #  data
                    buf[i] = int(self.readHexByte(in_))
                    cs += buf[i]
                cs += self.readHexByte(in_)
                #  checksum

                if (cs & 0xff) != 0:
                    raise ValueError("Checksum error")
                if b == 0:
                    #  data record
                    for i in range(0, lenfield):
                        if self.ihxData[addr + i] >= 0:
                            logging.warning('Memory at position ' + "%x" % i + " overwritten")
                        self.ihxData[addr + i] = int((buf[i] & 255))
                elif b == 1:
                    #  eof record
                    eof = True
                else:
                    raise ValueError("Invalid record type: " + str(b))
        except ValueError as e:
            # raise IhxFileDamagedException(fileName, line, e.getLocalizedMessage())
            raise ValueError("Some Problem with decoding IHX file :(: %s" % str(e))
        try:
            in_.close()
        except Exception as e:
            logging.error('Error closing file ' + fname + ": " + str(e))

    #  ******* dataInfo ************************************************************
    #
    #      * Print out some information about the memory usage.
    #      * @param out Where the data is printed out.
    #
    def dataInfo(self, out):
        """ generated source for method dataInfo """
        addr = -1
        i = 0
        while i <= 65536:
            #  data
            if (i == 65536 or self.ihxData[i] < 0) and addr >= 0:
                logging.info(i - addr + " Bytes from " + "%x" % addr + " to " + "%x" % (i - 1))
                addr = -1
            if i < 65536 and self.ihxData[i] >= 0 and addr < 0:
                addr = i
            i += 1


class Ztex1v1(object):

    CAPABILITY_FPGA = 1

    def __init__(self):
        self.dev = None
        self.firmwareProgrammed = False
        self.fpgaProgrammed = False
        self.deviceInfo = None

    def checkCapability(self, i):
        if self.deviceInfo:
            if self.deviceInfo["interfaceCapabilities"][i // 8] & (1 << (i % 8)) != 0:
                return True

        raise IOError("ZTEX: Device does not have required capability: %d" % i)
        # return False

    def vendorRequest2(self, cmd, name="Unknown", value=0, index=0, maxlen=0):
        data = self.dev.ctrl_transfer(0xc0, cmd, value, index, maxlen, 500)
        if len(data) != maxlen:
            raise IOError("ZTEX USB: %s: Expected %d bytes, received %d" % (name, maxlen, len(data)))
        return data

    def vendorCommand(self, cmd, name="Unknown", value=0, index=0, data=[]):
        self.dev.ctrl_transfer(0x40, cmd, value, index, data, 200)

    def getFpgaState(self):
        """ generated source for method getFpgaState """
        self.checkCapability(self.CAPABILITY_FPGA)
        buffer_ = self.vendorRequest2(0x30, "getFpgaState", maxlen=9)
        self.fpgaConfigured = buffer_[0] == 0
        self.fpgaChecksum = buffer_[1] & 0xff
        self.fpgaBytes = ((buffer_[5] & 0xff) << 24) | ((buffer_[4] & 0xff) << 16) | ((buffer_[3] & 0xff) << 8) | (buffer_[2] & 0xff)
        self.fpgaInitB = buffer_[6] & 0xff
        self.fpgaFlashResult = buffer_[7]
        self.fpgaFlashBitSwap = buffer_[8] != 0


    def probe(self, verbose=False):
        self.dev = None
        self.firmwareProgrammed = False
        self.fpgaProgrammed = False
        self.deviceInfo = None

        # Find unprogrammed chips
        dev = usb.core.find(idVendor=cypressVendorId, idProduct=cypressProductId)
        if dev:
            self.dev = dev
            self.firmwareProgrammed = False
            if verbose: logging.info('Found unprogrammed device')

        # Find programmed chips
        dev = usb.core.find(idVendor=ztexVendorId, idProduct=ztexProductId)
        if dev:
            self.dev = dev
            self.firmwareProgrammed = True
            if verbose: logging.info('Found programmed device')

            # Probe the device to determine additional information
            buf = self.dev.ctrl_transfer(0xc0, 0x22, 0, 0, 40, 500)

            self.deviceInfo = {
                               'productId':list(buf[6:10]),
                               'fwVersion':buf[10],
                               'interfaceVersion':buf[11],
                               'interfaceCapabilities':list(buf[12:18]),
                               'moduleReserved': list(buf[18:30])}

        if not self.dev:
            raise Warning("Failed to find USB Device")

    def resetFpga(self):
        """ generated source for method resetFpga """
        self.checkCapability(self.CAPABILITY_FPGA)
        self.vendorCommand(0x31, "resetFpga")

    def configureFpgaLS(self, fwFileLike, force=False, bs=-1):
        """ generated source for method configureFpgaLS """
        transactionBytes = 2048  # 256 if certainWorkarounds else 2048
        t0 = 0
        self.checkCapability(self.CAPABILITY_FPGA)
        if self.fpgaConfigured:
            raise IOError("Attempting to configure FPGA when already configured...")

        buffer_ = [None] * (16 * 1024 * 1024 / transactionBytes)
        size = 0
        try:
            # Read entire thing in
            #with fwFileLike as f: inputStream = f.read()
            inputStream = fwFileLike.read()
            fwFileLike.close()
            streamCnt = 0

            j = transactionBytes
            for i in range(0, len(buffer_)):
                if j != transactionBytes: break
                buffer_[i] = bytearray(inputStream[streamCnt:(streamCnt + transactionBytes)])
                streamCnt += transactionBytes

                j = len(buffer_[i])

                if j < transactionBytes and j % 64 == 0:
                    j += 1
                size += j

                # if j < transactionBytes and j % 64 == 0:
                #    j += 1
                # size += j

        except IOError as e:
            # raise BitstreamReadException(e.getLocalizedMessage())
            logging.error('Shit - IO Error in configureFpgaLS')

        if size < 64 or size % 64 == 0:
            raise ValueError("Invalid file size: " + str(size))
        if bs < 0 or bs > 1:
            bs = self.detectBitstreamBitOrder(buffer_[0])
        if bs == 1:
            self.swapBits(buffer_, size)
        tries = 10

        while tries > 0:
            self.resetFpga()
            try:
                # t0 = -Date().getTime()
                bs = 0
                cs = 0

                for i in range(0, len(buffer_)):
                    if  i * transactionBytes >= size:
                        break
                    j = size - i * transactionBytes
                    if j > transactionBytes:
                        j = transactionBytes
                    self.vendorCommand(0x32, "sendFpgaData", 0, 0, buffer_[i])

                    bs += j
                    for k in range(0, len(buffer_[i])):
                        cs = (cs + (buffer_[i][k] & 0xff)) & 0xff

                self.getFpgaState()
                if not self.fpgaConfigured:
                    raise IOError("FPGA configuration failed: DONE pin does not go high (size=" + self.fpgaBytes + " ,  " + (bs - self.fpgaBytes) + " bytes got lost;  checksum=" + self.fpgaChecksum + " , should be " + cs + ";  INIT_B_HIST=" + self.fpgaInitB + ")")

                #if self.enableExtraFpgaConfigurationChecks:
                #    if self.fpgaBytes != 0 and self.fpgaBytes != bs:
                #        System.err.println("Warning: Possible FPGA configuration data loss: " + (bs - self.fpgaBytes) + " bytes got lost")
                #    if self.fpgaInitB != 222:
                #        System.err.println("Warning: Possible Bitstream CRC error: INIT_B_HIST=" + self.fpgaInitB)
                tries = 0
                #t0 += Date().getTime()
            except IOError as e:
                if tries > 1:
                    logging.warning(str(e) + ": Retrying it ...")
                else:
                    raise
            tries -= 1
        time.sleep(0.1)
        return t0

    def detectBitstreamBitOrder(self, buf):
        """ generated source for method detectBitstreamBitOrder """
        i = 0
        while i < len(buf):
            if ((buf[i] & 255) == 0xaa) and ((buf[i + 1] & 255) == 0x99) and ((buf[i + 2] & 255) == 0x55) and ((buf[i + 3] & 255) == 0x66):
                return 1
            if ((buf[i] & 255) == 0x55) and ((buf[i + 1] & 255) == 0x99) and ((buf[i + 2] & 255) == 0xaa) and ((buf[i + 3] & 255) == 0x66):
                return 0
            i += 1
        logging.warning('Unable to determine bitstream bit order: no signature found')
        return 0

    #  ******* swapBits ************************************************************
    def swapBits(self, buf, size):
        """ generated source for method swapBits """
        j = 0
        k = 0
        i = 0
        for i in range(0, size):
            while k >= len(buf[j]):
                j += 1
                k = 0
            b = buf[j][k]
            buf[j][k] = int((((b & 128) >> 7) | ((b & 64) >> 5) | ((b & 32) >> 3) | ((b & 16) >> 1) | ((b & 8) << 1) | ((b & 4) << 3) | ((b & 2) << 5) | ((b & 1) << 7)))
            k += 1

    def uploadFirmware(self, ihxFile):
        """ generated source for method uploadFirmware """
        ez = EzUsb(self.dev)
        time = ez.uploadFirmware(ihxFile)
        return time


if __name__ == '__main__':
    ztex = Ztex1v1()
    ztex.probe(True)

    if ztex.firmwareProgrammed == False:
        f = IhxFile("OpenADC.ihx")
        ztex.uploadFirmware(f)
        time.sleep(1)
        ztex.probe()
    else:
        print("ZTEX Firmware upload skipped")

    if ztex.deviceInfo["interfaceVersion"] != 1:
        raise IOError("Unknown interface version, invalid ZTEX Firmware?. Device info: %s" % str(ztex.deviceInfo))

    if ztex.deviceInfo["productId"] != [10, 12, 0, 0]:
        raise IOError("Unknown productId, invalid ZTEX Firmware/Module?. Device info: %s" % str(ztex.deviceInfo))

    ztex.getFpgaState()
    if ztex.fpgaConfigured == False:
        print("Programming FPGA...")
        ztex.configureFpgaLS(open(r"C:\E\Documents\academic\sidechannel\chipwhisperer\hardware\capture\chipwhisperer-rev2\hdl\ztex_rev2_1.11c_ise\interface.bit", "rb"))

    else:
        print("FPGA Configuration skipped")



# Authors: Colin O'Flynn
# HIGHLEVEL_CLASSLOAD_FAIL_FUNC_DEBUG
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging
import sys
from .. import _qt as openadc_qt
from chipwhisperer.common.utils.parameter import Parameterized, Parameter, setupSetParam
try:
    import ftd2xx as ft
except:
    ft = None


class OpenADCInterface_FTDI(Parameterized):
    _name = "FTDI (SASEBO-W/SAKURA-G)"

    def __init__(self, oadcInstance):
        self.serialNumber = ''
        self._serialnumbers = ['']

        self.params = Parameter(name=self.getName(), type='group')
        self.params.addChildren([
            {'name':'Refresh Device List', 'type':'action', 'action':self.serialRefresh},
            {'name':'Device Serial Number', 'key':'snum', 'type':'list', 'values':[''], 'get':self.getSerialNumber, 'set':self.setSelectedDevice},
        ])
        self.ser = None

        if (openadc_qt is None) or (ft is None):
            raise ImportError("Needed imports for FTDI missing")
        else:
            self.scope = oadcInstance

    def getSerialNumber(self):
        return self.serialNumber

    @setupSetParam("Device Serial Number")
    def setSelectedDevice(self, snum):
        self.serialNumber = snum

    def con(self):
        if self.ser is None:
            try:
                self.dev = ft.openEx(str(self.serialNumber), ft.ftd2xx.OPEN_BY_SERIAL_NUMBER)
                self.dev.setBitMode(0x00, 0x40)
                self.dev.setTimeouts(500, 500)
                self.dev.setLatencyTimer(2)
                self.ser = self
            except ft.ftd2xx.DeviceError as e:
                self.ser = None
                raise IOError("Could not open %s: %s" % (self.serialNumber, e))

        try:
            self.scope.con(self.ser)
            logging.info('OpenADC Found, Connecting')
        except IOError as e:
            exctype, value = sys.exc_info()[:2]
            raise IOError("OpenADC Error: %s" % (str(exctype) + str(value)) + " - " + e.message)

    def dis(self):
        self.ser = None
        if hasattr(self, 'dev'):
            self.dev.close()
            del self.dev

    def __del__(self):
        self.dis()

    def serialRefresh(self, _=None):
        serialnames = ft.listDevices()
        if serialnames == None:
            serialnames = [""]
        self.setSerialNumberLimits(serialnames)
        self.findParam('snum').setValue(serialnames[0])

    def setSerialNumberLimits(self, newitems):
        for s in newitems:
            if s not in self._serialnumbers:
                self._serialnumbers.append(s)

        self.findParam('snum').setLimits(self._serialnumbers)

    def read(self, N=0, debug=False):
        return bytearray(self.dev.read(N))

    def write(self, data, debug=False):
        return self.dev.write(data)



# -*- coding: utf-8 -*-
"""
.. module:: chipwhisperer
   :platform: Unix, Windows
   :synopsis: Test

.. moduleauthor:: NewAE Technology Inc.

Main module for ChipWhisperer.
"""
import os, os.path, time
import warnings
from zipfile import ZipFile

from chipwhisperer.capture import scopes, targets
from chipwhisperer.capture.api import programmers
from chipwhisperer.capture import acq_patterns as key_text_patterns
from chipwhisperer.common.utils.util import camel_case_deprecated
from chipwhisperer.common.api import ProjectFormat as project
from chipwhisperer.common.traces import Trace
from chipwhisperer.common.utils import util
from chipwhisperer.capture.scopes.cwhardware.ChipWhispererSAM3Update import SAMFWLoader

# replace bytearray with inherited class with better repr and str.
import builtins
builtins.bytearray = util.bytearray

# from chipwhisperer.capture.scopes.cwhardware import ChipWhispererSAM3Update as CWFirmwareUpdate

ktp = key_text_patterns #alias


def program_target(scope, prog_type, fw_path, **kwargs):
    """Program the target using the programmer <type>

    Programmers can be found in the programmers submodule

    Args:
       scope (ScopeTemplate): Connected scope object to use for programming
       prog_type (Programmer): Programmer to use. See chipwhisperer.programmers
           for available programmers
       fw_path (str): Path to hex file to program

    .. versionadded:: 5.0.1
        Simplified programming target
    """
    if prog_type is None: #[makes] automating notebooks much easier
        return
    prog = prog_type(**kwargs)
    prog.scope = scope
    prog._logging = None
    prog.open()
    prog.find()
    prog.erase()
    prog.program(fw_path, memtype="flash", verify=True)
    prog.close()


programTarget = camel_case_deprecated(program_target)


def open_project(filename):
    """Load an existing project from disk.

    Args:
       filename (str): Path to project file.

    Returns:
       A chipwhisperer project object.

    Raises:
       OSError: filename does not exist.
    """
    filename = project.ensure_cwp_extension(filename)

    proj = project.Project()
    proj.load(filename)
    return proj


openProject = camel_case_deprecated(open_project)


def create_project(filename, overwrite=False):
    """Create a new project with the path <filename>.

    If <overwrite> is False, raise an OSError if this path already exists.

    Args:
       filename (str): File path to create project file at. Must end with .cwp
       overwrite (bool, optional): Whether or not to overwrite an existing
           project with <filename>. Raises an OSError if path already exists
           and this is false. Defaults to false.

    Returns:
       A chipwhisperer project object.

    Raises:
       OSError: filename exists and overwrite is False.
    """
    filename = project.ensure_cwp_extension(filename)

    if os.path.isfile(filename) and (overwrite == False):
        raise OSError("File " + filename + " already exists")

    # If the user gives a relative path including ~, expand to the absolute path
    filename = os.path.abspath(os.path.expanduser(filename))

    proj = project.Project()
    proj.setFilename(filename)

    return proj


createProject = camel_case_deprecated(create_project)


def import_project(filename, file_type='zip', overwrite=False):
    """Import and open a project.

    Will import the **filename** by extracting to the current working
    directory.

    Currently support file types:
     * zip

    Args:
        filename (str): The file name to import.
        file_type (str): The type of file that is being imported.
            Default is zip.
        overwrite (bool): Whether or not to overwrite the project given as
            the **import_as** project.

    .. versionadded:: 5.1
        Add **import_project** function.
    """
    # extract name from input file
    input_dir, input_file = os.path.split(filename)
    input_file_root, input_file_ext = os.path.splitext(input_file)
    input_abs_path = os.path.abspath(filename)

    # use the appropriate type of import
    if file_type == 'zip':
        with ZipFile(input_abs_path, 'r') as project_zip:
            output_path = None
            for path in project_zip.namelist():
                root, ext = os.path.splitext(path)
                if ext == '.cwp':
                    directory, project_name = os.path.split(root)
                    output_path = ''.join([project_name, '.cwp'])

                    # check if name already exists in projects
                    if os.path.isfile(output_path) and (overwrite == False):
                        raise OSError("File " + output_path + " already exists")

                    # extract the project.cwp file and project_data directory to
                    # the PROJECT_DIR
                    project_zip.extractall(path=os.getcwd())

            if output_path is None:
                raise ValueError('Zipfile does not contain a .cwp file, so it cannot be imported')
    else:
        raise ValueError('Import from file type not supported: {}'.format(file_type))

    proj = project.Project()
    proj.load(output_path)

    return proj


def scope(scope_type=None, sn=None):
    """Create a scope object and connect to it.

    This function allows any type of scope to be created. By default, the
    object created is based on the attached hardware (OpenADC for
    CWLite/CW1200, CWNano for CWNano).

    Scope Types:
     * :class:`scopes.OpenADC` (Pro and Lite)
     * :class:`scopes.CWNano` (Nano)

    If multiple chipwhisperers are connected, the serial number of the one you
    want to connect to can be specified by passing sn=<SERIAL_NUMBER>

    Args:
       scope_type (ScopeTemplate, optional): Scope type to connect to. Types
           can be found in chipwhisperer.scopes. If None, will try to detect
           the type of ChipWhisperer connected. Defaults to None.
       sn (str, optional): Serial number of ChipWhisperer that you want to
           connect to. Required if more than one ChipWhisperer
           of the same type is connected (i.e. two CWNano's or a CWLite and
           CWPro). Defaults to None.

    Returns:
        Connected scope object.

    Raises:
        OSError: Can be raised for issues connecting to the chipwhisperer, such
            as not having permission to access the USB device or no ChipWhisperer
            being connected.
        Warning: Raised if multiple chipwhisperers are connected, but the type
            and/or the serial numbers are not specified

    .. versionchanged:: 5.1
        Added autodetection of scope_type
    """
    from chipwhisperer.common.utils.util import get_cw_type
    if scope_type is None:
        scope_type = get_cw_type(sn)
    scope = scope_type()
    scope.con(sn)
    return scope


def target(scope, target_type=targets.SimpleSerial, **kwargs):
    """Create a target object and connect to it.

    Args:
       scope (ScopeTemplate): Scope object that we're connecting to the target
           through.
       target_type (TargetTemplate, optional): Target type to connect to.
           Defaults to targets.SimpleSerial. Types can be found in
           chipwhisperer.targets.
       **kwargs: Additional keyword arguments to pass to target setup. Rarely
           needed.

    Returns:
        Connected target object specified by target_type.
    """
    target = target_type()
    target.con(scope, **kwargs)
    return target

def capture_trace(scope, target, plaintext, key=None):
    """Capture a trace, sending plaintext and key

    Does all individual steps needed to capture a trace (arming the scope
    sending the key/plaintext, getting the trace data back, etc.)

    Args:
        scope (ScopeTemplate): Scope object to use for capture.
        target (TargetTemplate): Target object to read/write text from.
        plaintext (bytearray): Plaintext to send to the target. Should be
            unencoded bytearray (will be converted to SimpleSerial when it's
            sent). If None, don't send plaintext.
        key (bytearray, optional): Key to send to target. Should be unencoded
            bytearray. If None, don't send key. Defaults to None.

    Returns:
        :class:`Trace <chipwhisperer.common.traces.Trace>` or None if capture
        timed out.

    Raises:
        Warning or OSError: Error during capture.

    Example:
        Capturing a trace::

            import chipwhisperer as cw
            scope = cw.scope()
            scope.default_setup()
            target = cw.target()
            ktp = cw.ktp.Basic()
            key, pt = ktp.new_pair()
            trace = cw.capture_trace(scope, target, pt, key)

    .. versionadded:: 5.1
        Added to simplify trace capture.
    """
    if key:
        target.set_key(key)

    scope.arm()

    if plaintext:
        target.simpleserial_write('p', plaintext)

    ret = scope.capture()

    i = 0
    while not target.is_done():
        i += 1
        time.sleep(0.05)
        if i > 100:
            warnings.warn("Target did not finish operation")
            return None

    if ret:
        warnings.warn("Timeout happened during capture")
        return None

    response = target.simpleserial_read('r', 16)
    wave = scope.get_last_trace()

    return Trace(wave, plaintext, response, key)


captureTrace = camel_case_deprecated(capture_trace)





# -*- coding: utf-8 -*-
#
# Original Author: Ivan A-R <ivan@tuxotronic.org>
# Modified by: Colin O'Flynn (2017-2018), NewAE Technology Inc.
# Original Project page: http://tuxotronic.org/wiki/projects/stm32loader
#
# This file is part of stm32loader.
#
# stm32loader is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# stm32loader is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License
# along with stm32loader; see the file COPYING3.  If not see
# <http://www.gnu.org/licenses/>.
#==========================================================================

import logging
import os
import struct
import time
import traceback
from datetime import datetime
from chipwhisperer.capture.utils.IntelHex import IntelHex
from chipwhisperer.common.utils.timer import nonBlockingDelay
from functools import reduce

#From ST AN2606, See Section 50 (Device-dependent bootloader parameters), Page 244/268 on Rev 30 of document
#http://www.st.com/content/ccc/resource/technical/document/application_note/b9/9b/16/3a/12/1e/40/0c/CD00167594.pdf/files/CD00167594.pdf/jcr:content/translations/en.CD00167594.pdf

class STM32FDummy(object):
    signature = 0x000
    name = "Unknown STM32F"

class STM32F071(object):
    signature = 0x448
    name = "STM32F071xx/STM32F072xx"

class STM32F10xxx_LD(object):
    signature = 0x412
    name = "STM32F10xxx Low-density"

class STM32F10xxx_MD(object):
    signature = 0x410
    name = "STM32F10xxx Medium-density"

class STM32F10xxx_HD(object):
    signature = 0x414
    name = "STM32F10xxx High-density"

class STM32F10xxx_XL(object):
    signature = 0x416
    name = "STM32F10xxx XL-density"

class STM32F10xxx_MDV(object):
    signature = 0x420
    name = "STM32F10xxx Medium-density value line"

class STM32F10xxx_HDV(object):
    signature = 0x428
    name = "STM32F10xxx High-density value line"

class STM32F2(object):
    signature = 0x411
    name = "STM32F2"

class STM32F303cBC(object):
    signature = 0x422
    name = "STM32F302xB(C)/303xB(C)"

class STM32F40xxx(object):
    signature = 0x413
    name = "STM32F40xxx/41xxx"

supported_stm32f = [STM32F071(), STM32F10xxx_LD(), STM32F10xxx_MD(), STM32F10xxx_HD(), STM32F10xxx_XL(), STM32F10xxx_MDV(),
                    STM32F10xxx_HDV(), STM32F2(), STM32F303cBC(), STM32F40xxx()]

def print_fun(s):
    print(s)

class CmdException(Exception):
    pass

class STM32FSerial(object):
    """
    Class for programming an STM32F device using a serial port or ChipWhisperer-Serial
    """

    def __init__(self, cwserial=None, cwapi=None, spname=None, timeout=200, slow_speed=False, small_blocks=False):
        """
        Set the communications instance.
        """

        self._cwapi = cwapi
        self.scope = None # set with the programmer (needed for non-gui compatibility)
        self._cwserial = cwserial
        self._timeout = timeout
        self._chip = STM32FDummy()
        self.lastFlashedFile = "unknown"
        self.extended_erase = 0
        self.slow_speed = slow_speed
        self.small_blocks = small_blocks

        self._old_baud = None

    def open_port(self, baud=115200):

        if self._cwserial:
            self._old_baud = self._cwserial._baud
            self._old_parity  = self._cwserial._parity
            self._old_stopbits = self._cwserial._stopbits
            self._old_timeout = self._cwserial.timeout
            self._cwserial.init(baud, stopbits=1, parity="even")
            self._cwserial.timeout = self._timeout

            self.sp = self._cwserial
        else:
            raise NotImplementedError("CW-Serial is only supported method (for now)")

    def close_port(self):
        if self._old_baud:
            self._cwserial.init(baud=self._old_baud, stopbits=self._old_stopbits, parity=self._old_parity)
            self._cwserial.timeout = self._old_timeout
        self._old_baud = None
        self._old_parity = None
        self._old_stopbits = None

        if self._cwapi:
            self._cwapi.setParameter(['CW Extra Settings', 'Target IOn GPIO Mode', 'nRST: GPIO', 'Default'])

        if self.scope:
            self.scope.io.nrst = 'default'

        if hasattr(self.sp, "close"):
            self.sp.close()

    def find(self, logfunc=print_fun):
        #setup serial port (or CW-serial port?)

        if logfunc is None:
            logfunc = print_fun

        try:
            self.initChip()
        except IOError:
            logfunc("Failed to detect chip. Check following: ")
            logfunc("   1. Connections and device power. ")
            logfunc("   2. Device has valid clock (or remove clock entirely for internal osc).")
            logfunc("   3. On Rev -02 CW308T-STM32Fx boards, BOOT0 is routed to PDIC.")
            raise

        boot_version = self.cmdGet()
        chip_id = self.cmdGetID()

        for t in supported_stm32f:
            if chip_id == t.signature:
                logfunc("Detected known STMF32: %s" % t.name)
                self.setChip(t)
                return chip_id, t
        logfunc("Detected unknown STM32F ID: 0x%03x" % chip_id)
        return chip_id, None

    def program(self, filename, memtype="flash", verify=True, logfunc=print_fun, waitfunc=None):
        """Programs memory type, dealing with opening filename as either .hex or .bin file"""
        self.lastFlashedFile = filename

        f = IntelHex(filename)

        fsize = f.maxaddr() - f.minaddr()
        fdata = f.tobinarray(start=f.minaddr())
        startaddr = f.minaddr()

        logfunc("Attempting to program %d bytes at 0x%x"% (fsize, startaddr))

        logfunc("STM32F Programming %s..." % memtype)
        if waitfunc: waitfunc()
        self.writeMemory(startaddr, fdata)  # , erasePage=True

        logfunc("STM32F Reading %s..." % memtype)
        if waitfunc: waitfunc()

        try:
            self.verifyMemory(startaddr, fdata, self.small_blocks)
        except CmdException:
            logfunc("Error during verify. Retrying with small blocks...")
            self.verifyMemory(startaddr, fdata, True)

        logfunc("Verified %s OK, %d bytes" % (memtype, fsize))


    def autoProgram(self, hexfile, erase=True, verify=True, logfunc=print_fun, waitfunc=None):
        # Helper function for programmer UI
        # Automatically program device with some error checking

        status = "FAILED"

        fname = hexfile
        if logfunc: logfunc("***Starting FLASH program process at %s***" % datetime.now().strftime('%H:%M:%S'))
        if waitfunc: waitfunc()
        if os.path.isfile(fname):
            if logfunc: logfunc("File %s last changed on %s" % (fname, time.ctime(os.path.getmtime(fname))))
            if waitfunc: waitfunc()

            try:
                if logfunc: logfunc("Entering Programming Mode")
                if waitfunc: waitfunc()
                self.open_port()
                self.find(logfunc)

                if erase:
                    self.cmdEraseMemory()
                    logfunc("Erasing memory")

                if waitfunc: waitfunc()
                self.program(hexfile, memtype="flash", verify=verify, logfunc=logfunc, waitfunc=waitfunc)
                if waitfunc: waitfunc()
                if logfunc: logfunc("Exiting programming mode")
                self.close_port()
                if waitfunc: waitfunc()
                self.releaseChip()

                status = "SUCCEEDED"

            except IOError as e:
                if logfunc: logfunc("FAILED: %s" % str(e))
                try:
                    self.close_port()
                except IOError:
                    pass

        else:
            if logfunc: logfunc("%s does not appear to be a file, check path" % fname)

        if logfunc: logfunc("***FLASH Program %s at %s***" % (status, datetime.now().strftime('%H:%M:%S')))

        return status == "SUCCEEDED"

    def delay_func(self, ms_delay):
        """If using outside of GUI, can override this function"""
        return
        #nonBlockingDelay(ms_delay)

    def setChip(self, chiptype):
        self._chip = chiptype

    def reset(self):
        if self._cwapi:
            self._cwapi.setParameter(['CW Extra Settings', 'Target IOn GPIO Mode', 'nRST: GPIO', 'Low'])
            self.delay_func(10)
            self._cwapi.setParameter(['CW Extra Settings', 'Target IOn GPIO Mode', 'nRST: GPIO', 'High'])
            self.delay_func(25)
        elif self.scope:
            self.scope.io.nrst = 'low'
            time.sleep(0.010)
            self.scope.io.nrst = 'high'
            time.sleep(0.025)
        else:
            raise ValueError('requires either scope or api to be set')

    def set_boot(self, enter_bootloader):
        if enter_bootloader:
            if self._cwapi:
                self._cwapi.setParameter(['CW Extra Settings', 'Target IOn GPIO Mode', 'PDIC: GPIO', 'High'])
            elif self.scope:
                self.scope.io.pdic = 'high'
            else:
                raise ValueError('requires either scope or api to be set')
        else:
            if self._cwapi:
                self._cwapi.setParameter(['CW Extra Settings', 'Target IOn GPIO Mode', 'PDIC: GPIO', 'Low'])
            elif self.scope:
                self.scope.io.pdic = 'low'
            else:
                raise ValueError('requires either scope or api to be set')
        logging.info("Assuming appropriate BOOT pins set HIGH on STM32F Hardware now")


    def _wait_for_ask(self, info=""):
        # wait for ask
        try:
            ask = self.sp.read(1)[0]
        except:
            raise CmdException("Can't read port or timeout (%s). Target didn't respond when an ack was expected." % traceback.format_exc())
        else:
            if ask == 0x79:
                # ACK
                return 1
            else:
                if ask == 0x1F:
                    # NACK
                    raise CmdException("NACK " + info)
                else:
                    # Unknown response
                    raise CmdException("Unknown response. " + info + ": " + hex(ask))

    def initChip(self):
        self.set_boot(True)
        self.reset()
        fails = 0
        while fails < 5:
            try:
                #First 2-times, try resetting. After that don't in case reset is causing garbage on lines.
                if fails < 2:
                    self.reset()
                try:
                    self.sp.flush()
                    self.sp.write("\x7F")
                except AttributeError:
                    raise AttributeError('sp attribute requires call to open_port')
                return self._wait_for_ask("Syncro")
            except CmdException:
                logging.info("Sync failed with error %s, retrying..." % traceback.format_exc())
                fails += 1

        self.releaseChip()
        raise IOError("Could not detect STM32F, check connections, BOOT MODE entry setup")

    def releaseChip(self):
        self.set_boot(False)
        self.reset()

    def cmdGeneric(self, cmd):
        self.sp.write(chr(cmd))
        self.sp.write(chr(cmd ^ 0xFF))  # Control byte
        return self._wait_for_ask(hex(cmd))

    def cmdGet(self):
        if self.cmdGeneric(0x00):
            logging.info("*** Get command");
            len = self.sp.read(1)[0]
            version = self.sp.read(1)[0]
            logging.info("    Bootloader version: " + hex(version))
            #dat = map(lambda c: hex(self.sp.read(len))
            dat = list(map(hex, self.sp.read(len)))
            if '0x44' in dat:
                self.extended_erase = 1
            else:
                self.extended_erase = 0
            logging.info("    Available commands: " + ", ".join(dat))
            self._wait_for_ask("0x00 end")
            return version
        else:
            raise CmdException("Get (0x00) failed")

    def cmdGetVersion(self):
        if self.cmdGeneric(0x01):
            logging.debug("*** GetVersion command")
            version = self.sp.read(1)[0]
            self.sp.read(2)
            self._wait_for_ask("0x01 end")
            logging.debug("    Bootloader version: " + hex(version))
            return version
        else:
            raise CmdException("GetVersion (0x01) failed")

    def cmdGetID(self):
        if self.cmdGeneric(0x02):
            logging.debug("*** GetID command")
            len = self.sp.read(1)[0]
            id = self.sp.read(len + 1)
            self._wait_for_ask("0x02 end")
            return reduce(lambda x, y: x * 0x100 + y, id)
        else:
            raise CmdException("GetID (0x02) failed")

    def _encode_addr(self, addr):
        baddr = bytearray(struct.pack(">L", addr))
        crc = reduce(lambda x, y: x ^ y, baddr)
        baddr.append(crc)
        return bytes(baddr)

    def cmdReadMemory(self, addr, lng):
        assert (0 < lng <= 256)
        if self.cmdGeneric(0x11):
            logging.debug("*** ReadMemory command")
            self.sp.write(self._encode_addr(addr))
            self._wait_for_ask("0x11 address failed")
            N = (lng - 1) & 0xFF
            crc = N ^ 0xFF
            self.sp.write(chr(N) + chr(crc))
            self._wait_for_ask("0x11 length failed")
            return [c for c in self.sp.read(lng)]
        else:
            raise CmdException("ReadMemory (0x11) failed")

    def cmdGo(self, addr):
        if self.cmdGeneric(0x21):
            logging.debug("*** Go command")
            self.sp.write(self._encode_addr(addr))
            self._wait_for_ask("0x21 go failed")
        else:
            raise CmdException("Go (0x21) failed")

    def cmdWriteMemory(self, addr, data):
        padding = len(data) & 3
        if padding:
            data += [0xff, 0xff, 0xff, 0xff][padding:]
        assert (0 < len(data) <= 256)
        if self.cmdGeneric(0x31):
            logging.debug("*** Write memory command")
            self.sp.write(self._encode_addr(addr))
            self._wait_for_ask("0x31 address failed")
            # map(lambda c: hex(ord(c)), data)
            lng = (len(data) - 1) & 0xFF
            logging.debug("    %s bytes to write" % [lng + 1])
            self.sp.write(chr(lng))  # len really
            crc = lng
            for c in data:
                crc ^= c
                self.sp.write(chr(c))
                if self.slow_speed:
                    self.delay_func(5)
            self.sp.write(chr(crc))
            self._wait_for_ask("0x31 programming failed")
            logging.debug("    Write memory done")
        else:
            raise CmdException("Write memory (0x31) failed")

    def cmdEraseMemory(self, sectors=None):
        if self.extended_erase:
            return self.cmdExtendedEraseMemory()

        if self.cmdGeneric(0x43):
            logging.debug("*** Erase memory command")
            if sectors is None:
                # Global erase
                self.sp.write(chr(0xFF))
                self.sp.write(chr(0x00))
            else:
                # Sectors erase
                self.sp.write(chr((len(sectors) - 1) & 0xFF))
                crc = 0xFF
                for c in sectors:
                    crc = crc ^ c
                    self.sp.write(chr(c))
                self.sp.write(chr(crc))
            self._wait_for_ask("0x43 erasing failed")
            logging.info("    Erase memory done")
        else:
            raise CmdException("Erase memory (0x43) failed")

    def cmdExtendedEraseMemory(self):
        if self.cmdGeneric(0x44):
            logging.debug("*** Extended Erase memory command")
            # Global mass erase
            self.sp.write(chr(0xFF))
            self.sp.write(chr(0xFF))
            # Checksum
            self.sp.write(chr(0x00))
            tmp = self.sp.timeout
            self.sp.timeout = 30000
            print("Extended erase (0x44), this can take ten seconds or more")
            self._wait_for_ask("0x44 erasing failed")
            self.sp.timeout = tmp
            logging.info("    Extended Erase memory done")
        else:
            raise CmdException("Extended Erase memory (0x44) failed")

    def cmdWriteProtect(self, sectors):
        if self.cmdGeneric(0x63):
            logging.info("*** Write protect command")
            self.sp.write(chr((len(sectors) - 1) & 0xFF))
            crc = 0xFF
            for c in sectors:
                crc = crc ^ c
                self.sp.write(chr(c))
            self.sp.write(chr(crc))
            self._wait_for_ask("0x63 write protect failed")
            logging.info("    Write protect done")
        else:
            raise CmdException("Write Protect memory (0x63) failed")

    def cmdWriteUnprotect(self):
        if self.cmdGeneric(0x73):
            logging.info("*** Write Unprotect command")
            self._wait_for_ask("0x73 write unprotect failed")
            self._wait_for_ask("0x73 write unprotect 2 failed")
            logging.info("    Write Unprotect done")
        else:
            raise CmdException("Write Unprotect (0x73) failed")

    def cmdReadoutProtect(self):
        if self.cmdGeneric(0x82):
            logging.info("*** Readout protect command")
            self._wait_for_ask("0x82 readout protect failed")
            self._wait_for_ask("0x82 readout protect 2 failed")
            logging.info("    Read protect done")
        else:
            raise CmdException("Readout protect (0x82) failed")

    def cmdReadoutUnprotect(self):
        if self.cmdGeneric(0x92):
            logging.info("*** Readout Unprotect command")
            self._wait_for_ask("0x92 readout unprotect failed")
            self._wait_for_ask("0x92 readout unprotect 2 failed")
            logging.info("    Read Unprotect done")
        else:
            raise CmdException("Readout unprotect (0x92) failed")


            # Complex commands section

    def verifyMemory(self, addr, fdata, smallblocks=False):

        fdata_idx = 0

        if smallblocks:
            block_size = 64
        else:
            block_size = 256

        lng = len(fdata)

        fails = 0

        data = []
        while lng > block_size:
            logging.debug("Read %(len)d bytes at 0x%(addr)X" % {'addr': addr, 'len': block_size})
            data = self.cmdReadMemory(addr, block_size)
            if self.slow_speed:
                self.delay_func(1)

            redo_block = False
            for i in range(0, len(data)):
                if fdata[i+fdata_idx] != data[i]:
                    fails += 1
                    logging.info("Verify failed at 0x%04x, %x != %x" % (i, fdata[i+fdata_idx], data[i]))
                    if fails > 3:
                        raise IOError("Verify failed at 0x%04x, %x != %x" % (i, fdata[i+fdata_idx], data[i]))
                    else:
                        #Redo this block
                        logging.info("Read error - attempting retry")
                        redo_block = True
                        break

            if redo_block:
                continue

            fails = 0

            fdata_idx += block_size
            addr += block_size
            lng -= block_size

        logging.debug("Read %(len)d bytes at 0x%(addr)X" % {'addr': addr, 'len': lng})
        fails = 0
        while lng:
            data = self.cmdReadMemory(addr, lng)
            block_size = lng

            redo_block = False
            for i in range(0, len(data)):
                if fdata[i + fdata_idx] != data[i]:
                    fails += 1
                    logging.info("Verify read failure in block at address 0x%04x" % (i + fdata_idx))
                    if fails > 3:
                        raise IOError("Verify repeated failure at 0x%04x, %x != %x" % (i + fdata_idx, fdata[i + fdata_idx], data[i]))
                    else:
                        # Redo this block
                        logging.info("Verify failure - attempting retry now")
                        redo_block = True
                        break

            if redo_block:
                continue

            fdata_idx += block_size
            addr += block_size
            lng -= block_size


    def readMemory(self, addr, lng, smallblocks=False):
        """Read from flash using bootloader. If smallblocks is true uses a smaller
           block size, which can be more reliable as sometimes the full block size
           causes weird timeouts"""

        if smallblocks:
            block_size = 64
        else:
            block_size = 256

        data = []
        while lng > block_size:
            logging.debug("Read %(len)d bytes at 0x%(addr)X" % {'addr': addr, 'len': block_size})
            data += self.cmdReadMemory(addr, block_size)
            if self.slow_speed:
                self.delay_func(1)
            addr += block_size
            lng -= block_size

        logging.debug("Read %(len)d bytes at 0x%(addr)X" % {'addr': addr, 'len': lng})
        if lng:
            data += self.cmdReadMemory(addr, lng)
        return data

    def writeMemory(self, addr, data, smallblocks=False):
        """Write to the memory on the chip. If smallblocks is true, it uses a smaller
           block size, which can be more reliable as sometimes the full block size
           causes weird timeouts"""

        lng = len(data)
        data = list(data)

        if smallblocks:
            block_size = 64
        else:
            block_size = 256

        offs = 0
        while lng > block_size:
            logging.debug("Write %(len)d bytes at 0x%(addr)X" % {'addr': addr, 'len': block_size})

            try:
                self.cmdWriteMemory(addr, data[offs:offs + block_size])
            except CmdException:
                # Try shrinking the block size for the writes
                block_size = 64
                logging.debug("Write with block size 256 failed, retrying with block size of 64")
                self.cmdWriteMemory(addr, data[offs:offs + block_size])

            if self.slow_speed:
                self.delay_func(1)
            offs += block_size
            addr += block_size
            lng -= block_size

        if lng:
            logging.debug("Write %(len)d bytes at 0x%(addr)X" % {'addr': addr, 'len': lng})
            self.cmdWriteMemory(addr, data[offs:])



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging

import random
from usb import USBError

import binascii
from ._base import TargetTemplate
from .simpleserial_readers.cwlite import SimpleSerial_ChipWhispererLite
from chipwhisperer.common.utils import util
from collections import OrderedDict
from chipwhisperer.common.utils.util import camel_case_deprecated


class SimpleSerial(TargetTemplate, util.DisableNewAttr):
    """SimpleSerial target object.

    This class contains the public API for a target that uses serial
    communication.

    The easiest way to connect to the target is::

        import chipwhisperer as cw
        scope = cw.scope()
        target = cw.target(scope)

    The target is automatically connected to if the default configuration
    adequate.

    For more help use the help() function with one of the submodules
    (target.baud, target.write, target.read, ...).

      * :attr:`target.baud <.SimpleSerial.baud>`
      * :meth:`target.write <.SimpleSerial.write>`
      * :meth:`target.read <.SimpleSerial.read>`
      * :meth:`target.simpleserial_wait_ack <.SimpleSerial.simpleserial_wait_ack>`
      * :meth:`target.simpleserial_write <.SimpleSerial.simpleserial_write>`
      * :meth:`target.simpleserial_read <.SimpleSerial.simpleserial_read>`
      * :meth:`target.set_key <.SimpleSerial.set_key>`
      * :meth:`target.close <.SimpleSerial.close>`
      * :meth:`target.con <.SimpleSerial.con>`

    """
    _name = "Simple Serial"

    def __init__(self):
        TargetTemplate.__init__(self)

        self.ser = SimpleSerial_ChipWhispererLite()

        self.keylength = 16
        self.textlength = 16
        self.outputlength = 16
        self.input = ""
        self.key = ""
        self._protver = 'auto'
        self._read_timeout = 500
        self.masklength = 18
        self._fixedMask = True
        self.initmask = '1F 70 D6 3C 23 EB 1A B8 6A D5 E2 0D 5F D9 58 A3 CA 9D'
        self._mask = util.hexStrToByteArray(self.initmask)
        self.protformat = 'hex'
        self.last_key = bytearray(16)

        # Preset lists are in the form
        # {'Dropdown Name':['Init Command', 'Load Key Command', 'Load Input Command', 'Go Command', 'Output Format']}
        # If a command is None, it's left unchanged and the text field is editable;
        # Otherwise, it's loaded with the value and set to readonly
        self.presets = {
            'Custom':[None, None, None, None, None],
            'SimpleSerial Encryption':['','k$KEY$\\n', '', 'p$TEXT$\\n', 'r$RESPONSE$\\n'],
            'SimpleSerial Authentication':['','k$KEY$\\n', 't$EXPECTED$\\n', 'p$TEXT$\\n', 'r$RESPONSE$\\n'],
            'Glitching':[None, None, None, None, '$GLITCH$\\n'],
        }
        self._preset = 'Custom'

        self._linkedmaskgroup = (('maskgroup', 'cmdmask'), ('maskgroup', 'initmask'), ('maskgroup', 'masktype'),
                                 ('maskgroup', 'masklen'), ('maskgroup', 'newmask'))



        self._proto_ver = "auto"
        self._proto_timeoutms = 20
        self._init_cmd = ''
        self._key_cmd = 'k$KEY$\n'
        self._input_cmd = ''
        self._go_cmd = 'p$TEXT$\n'
        self._output_cmd = 'r$RESPONSE$\n'

        self._mask_enabled = False
        self._mask_cmd = 'm$MASK$\n'

        self.outstanding_ack = False

        self.setConnection(self.ser)
        self.disable_newattr()

    def getInitialMask(self):
        return " ".join(["%02X" % b for b in self._mask])

    def setInitialMask(self, initialMask, binaryMask=False):
        if initialMask:
            if binaryMask:
                maskStr = ''
                for s in initialMask:
                    maskStr += '%02x' % s
                self._mask = bytearray(initialMask)
            else:
                maskStr = initialMask
                self._mask = util.hexStrToByteArray(initialMask)
            self.initmask = maskStr

    @property
    def fixed_mask(self):
        if self.getMaskEnabled():
            return self.getInitialMask()
        return ''

    @fixed_mask.setter
    def fixed_mask(self, m):
        self.setInitialMask(m)

    def _dict_repr(self):
        dict = OrderedDict()
        dict['key_len'] = self.key_len
        dict['input_len'] = self.input_len
        dict['output_len'] = self.output_len
        dict['mask_len'] = self.mask_len
        dict['read_timeout'] = self.read_timeout

        dict['init_cmd']    = self.init_cmd
        dict['key_cmd']  = self.key_cmd
        dict['input_cmd']   = self.input_cmd
        dict['go_cmd']   = self.go_cmd
        dict['output_cmd'] = self.output_cmd
        dict['mask_cmd'] = self.mask_cmd

        dict['mask_enabled'] = self.mask_enabled
        dict['mask_type'] = self.mask_type
        if dict['mask_type'] == 'fixed':
            dict['fixed_mask'] = self.fixed_mask

        dict['baud']     = self.baud
        dict['protver'] = self.protver
        return dict

    @property
    def key_len(self):
        """The length of the key (in bytes)"""
        return self.keyLen()

    @key_len.setter
    def key_len(self, length):
        self.setKeyLen(length)

    @property
    def input_len(self):
        """The length of the input to the crypto algorithm (in bytes)"""
        return self.textLen()

    @input_len.setter
    def input_len(self, length):
        self.setTextLen(length)

    @property
    def output_len(self):
        """The length of the output expected from the crypto algorithm (in bytes)"""
        return self.textLen()

    @output_len.setter
    def output_len(self, length):
        return self.setOutputLen(length)


    @property
    def read_timeout(self):
        """Timeout in mS on how long to wait for target to respond."""
        return self._read_timeout

    @read_timeout.setter
    def read_timeout(self, timeout):
        self._read_timeout = timeout

    @property
    def mask_len(self):
        """The length of the mask to send (in bytes)"""
        return self.maskLen()

    @mask_len.setter
    def mask_len(self, length):
        return self.setMaskLen(length)

    @property
    def init_cmd(self):
        """The command sent to the target before starting a capture.

        This value is a string that is sent to the target via the serial port.
        It can contain 4 special strings that are replaced during each capture:
        - "$KEY$": The encryption key
        - "$TEXT$": The text input
        - "$EXPECTED$": The expected result of the target's operation
        - "$MASK$": The mask used for masked-AES implementation
        These strings are replaced with ASCII values
        ex: k$KEY$ -> k0011223344556677

        Getter: Return the current init command

        Setter: Set a new init command
        """
        return self._init_cmd

    @init_cmd.setter
    def init_cmd(self, cmd):
        self._init_cmd = cmd

    @property
    def key_cmd(self):
        """The command used to send the key to the target.

        See init_cmd for details about special strings.

        Getter: Return the current key command

        Setter: Set a new key command
        """
        return self._key_cmd

    @key_cmd.setter
    def key_cmd(self, cmd):
        self._key_cmd = cmd

    @property
    def input_cmd(self):
        """The command used to send the text input to the target.

        See init_cmd for details about special strings.

        Getter: Return the current text input command

        Setter: Set a new text input command
        """
        return self._input_cmd

    @input_cmd.setter
    def input_cmd(self, cmd):
        self._input_cmd = cmd

    @property
    def go_cmd(self):
        """The command used to tell the target to start the operation.

        See init_cmd for details about special strings.

        Getter: Return the current text input command

        Setter: Set a new text input command
        """
        return self._go_cmd

    @go_cmd.setter
    def go_cmd(self, cmd):
        self._go_cmd = cmd

    @property
    def output_cmd(self):
        """The expected format of the output string.

        The output received from the target is compared to this string after
        capturing a trace. If the format doesn't match, an error is logged.

        This format string can contain two special strings:
          * "$RESPONSE$": If the format contains $RESPONSE$, then this part of
            the received text is converted to the output text (ciphertext or
            similar). The length of this response string is given in outputLen()
            and set by setOutputLen().
          * "$GLITCH$": If the format starts with $GLITCH$, then all output is
            redirected to the glitch explorer.

        Getter: Return the current output format

        Setter: Set a new output format
        """
        return self._output_cmd

    @output_cmd.setter
    def output_cmd(self, cmd):
        self._output_cmd = cmd

    @property
    def mask_cmd(self):
        """The command used to set a mask for the masked-AES implementation.
        This command might be ignored by unsupported targets.

        See init_cmd for details about special strings.

        Getter: Return the current mask command

        Setter: Set a new mask command
        """
        return self._mask_cmd

    @mask_cmd.setter
    def mask_cmd(self, cmd):
        self._mask_cmd = cmd

    @property
    def mask_type(self):
        """mask_type is either 'fixed' or 'random'."""
        return "fixed" if self.getMaskType() else "random"

    @mask_type.setter
    def mask_type(self, masktype):
        if masktype == 'fixed' or masktype == True:
            self._fixedMask = True
        elif masktype == 'random' or masktype == False:
            self._fixedMask = False
        else:
            raise ValueError('Invalid value for mask_type. Should be "fixed" or "random"')

    @property
    def mask_enabled(self):
        return self._mask_enabled

    @mask_enabled.setter
    def mask_enabled(self, enable):
        self._mask_enabled = enable

    @property
    def baud(self):
        """The current baud rate of the serial connection.

        :Getter: Return the current baud rate.

        :Setter: Set a new baud rate. Valid baud rates are any integer in the
            range [500, 2000000].

        Raises:
            AttributeError: Target doesn't allow baud to be changed.
        """
        if hasattr(self.ser, 'baud') and callable(self.ser.baud):
            return self.ser.baud()
        else:
            raise AttributeError("Can't access baud rate")

    @baud.setter
    def baud(self, new_baud):
        if hasattr(self.ser, 'baud') and callable(self.ser.baud):
            self.ser.setBaud(new_baud)
        else:
            raise AttributeError("Can't access baud rate")

    @property
    def protver(self):
        """Get the protocol version used for the target
        """
        return self._proto_ver

    @protver.setter
    def protver(self, value):
        """Set the protocol version used for the target ('1.1', '1.0', or 'auto')
        """
        self._proto_ver = value

    def setKeyLen(self, klen):
        """ Set key length in bytes """
        self.keylength = klen

    def keyLen(self):
        """ Return key length in bytes """
        return self.keylength

    def setMaskLen(self, mlen):
        self.masklength = mlen

    def maskLen(self):
        return self.masklength

    def setTextLen(self, tlen):
        """ Set plaintext length. tlen given in bytes """
        self.textlength = tlen

    def textLen(self):
        """ Return plaintext length in bytes """
        return self.textlength

    def setOutputLen(self, tlen):
        """ Set plaintext length in bytes """
        self.outputlength = tlen

    def outputLen(self):
        """ Return output length in bytes """
        return self.outputlength

    def setProtFormat(self, protformat):
        """ Set the protocol format used 'bin' or 'hex' """
        self.protformat = protformat

    def protFormat(self):
        """ Return the protocol format used 'bin' or 'hex' """
        return self.protformat

    def getConnection(self):
        return self.ser

    def setConnection(self, con):
        self.ser = con
        self.ser.connectStatus = self.connectStatus
        self.ser.selectionChanged()

    def _con(self, scope = None):
        if not scope or not hasattr(scope, "qtadc"): Warning("You need a scope with OpenADC connected to use this Target")
        self.outstanding_ack = False

        self.ser.con(scope)
        # 'x' flushes everything & sets system back to idle
        self.ser.write("xxxxxxxxxxxxxxxxxxxxxxxx")
        self.ser.flush()


    def close(self):
        if self.ser != None:
            self.ser.close()

    def getVersion(self):
        self.ser.flush()
        self.ser.write("v\n")
        data = self.ser.read(4, timeout=self._proto_timeoutms)

        if len(data) > 1 and data[0] == 'z':
            logging.info("SimpleSerial: protocol V1.1 detected")
            return '1.1'
        else:
            logging.info("SimpleSerial: protocol V1.0 detected")
            return '1.0'


    def init(self):
        self.ser.flush()
        ver = self.protver
        if ver == 'auto':
            self._protver = self.getVersion()
        else:
            self._protver = ver
        self.outstanding_ack = False

        self.runCommand(self._init_cmd)
        # If we use a fix mask, set it once at init
        if self._mask_enabled and self._mask_cmd and self.getMaskType():
            self._mask = self.checkMask(self._mask)
            self.runCommand(self._mask_cmd)

    def newRandMask(self, _=None):
        new_mask = [random.randint(0, 255) for _ in range(self.maskLen())]
        self._mask = bytearray(new_mask)

    def reinit(self):
        if self._mask_enabled and self._mask_cmd:
            # Only set a mask if it's random. Fixed mask is set by init()
            if not self.getMaskType():  # Random
                self.newRandMask()
                self._mask = self.checkMask(self._mask)
                self.runCommand(self._mask_cmd)

    def setModeEncrypt(self):
        pass

    def setModeDecrypt(self):
        pass

    def convertVarToString(self, var):
        if isinstance(var, str):
            return var

        sep = ""
        s = sep.join(["%02x"%b for b in var])
        return s

    def runCommand(self, cmdstr, flushInputBefore=True):
        if self.connectStatus==False:
            raise Warning("Can't write to the target while disconected. Connect to it first.")

        if cmdstr is None or len(cmdstr) == 0:
            return

        # Protocol version 1.1 waits for ACK - if we have outstanding ACK, wait now
        if self._protver == '1.1':
            if self.outstanding_ack:
                # TODO - Should be user-defined maybe
                data = self.ser.read(4, timeout=500)
                if len(data) > 1:
                    if data[0] != 'z':
                        logging.error("SimpleSerial: ACK ERROR, read %02x" % ord(data[0]))
                else:
                    logging.error("SimpleSerial: ACK ERROR, did not see anything - TIMEOUT possible!")
                self.outstanding_ack = False

        varList = [("$KEY$",self.key, "Hex Encryption Key"),
                   ("$TEXT$",self.input, "Input Plaintext"),
                   ("$MASK$",self._mask, "Mask"),
                   ("$EXPECTED$", self.getExpected(), "Expected Ciphertext")]

        newstr = cmdstr

        #Find variables to insert
        for v in varList:
            if v[1] is not None:
                newstr = newstr.replace(v[0], self.convertVarToString(v[1]))

        #This is dumb
        newstr = newstr.replace("\\n", "\n")
        newstr = newstr.replace("\\r", "\r")

        #print newstr
        try:
            if flushInputBefore:
                self.ser.flushInput()
            if self.protformat == "bin":
                newstr = binascii.unhexlify(newstr)
            self.ser.write(newstr)
        except USBError:
            self.dis()
            raise Warning("Error in the target. It may have been disconnected.")
        except Exception as e:
            self.dis()
            raise e
        if self._protver == '1.1':
            self.outstanding_ack = True

    def loadEncryptionKey(self, key):
        """ Updates encryption key on target.

        The key is updated in this object and sent to the target over serial.
        """
        self.key = key
        if self.key:
            self.runCommand(self._key_cmd)

    def loadInput(self, inputtext):
        """ Sends plaintext to target

        Also updates the internal plaintext
        """
        self.input = inputtext
        self.runCommand(self._input_cmd)

    def loadMask(self, mask):
        self.mask = mask
        self.runCommand(self._mask_cmd)

    def is_done(self):
        return True

    def readOutput(self):
        dataLen= self.outputlength*2

        fmt = self._output_cmd
        #This is dumb
        fmt = fmt.replace("\\n", "\n")
        fmt = fmt.replace("\\r", "\r")

        if len(fmt) == 0:
            return None

        if fmt.startswith("$GLITCH$"):

            try:
                databytes = int(fmt.replace("$GLITCH$",""))
            except ValueError:
                databytes = 64

            self.newInputData.emit(self.ser.read(databytes,timeout=self.read_timeout))
            return None

        dataLen += len(fmt.replace("$RESPONSE$", ""))
        expected = fmt.split("$RESPONSE$")

        #Read data from serial port
        response = self.ser.read(dataLen, timeout=self.read_timeout)

        # If the protocol format is bin convert is back to hex for handling by CW
        if self.protformat == "bin":
            response = binascii.hexlify(response.encode('latin1'))

        if len(response) < dataLen:
            logging.warning('Response length from target shorter than expected (%d<%d): "%s".' % (len(response), dataLen, response))
            return None

        #Go through...skipping expected if applicable
        #Check expected first

        #Is a beginning part
        if len(expected[0]) > 0:
            if response[0:len(expected[0])] != expected[0]:
                logging.warning("Response start doesn't match what was expected:")
                logging.warning("Got {}, Expected {} + data".format(response, expected[0]))
                logging.warning("Hex Version: %s" % (" ".join(["%02x" % ord(t) for t in response])))

                return None

        startindx = len(expected[0])

        #Is middle part?
        data = bytearray(self.outputlength)
        if len(expected) == 2:
            for i in range(0,self.outputlength):
                # when glitched, the target might send us corrupted data...
                try:
                    data[i] = int(response[(i * 2 + startindx):(i * 2 + startindx + 2)], 16)
                except ValueError as e:
                    logging.warning('ValueError: %s' % str(e))

            startindx += self.outputlength*2

        #Is end part?
        if len(expected[1]) > 0:
            if response[startindx:startindx+len(expected[1])] != expected[1]:
                logging.warning("Unexpected end to response:")
                logging.warning("Got: {}, Expected {}".format(response, expected[1]))
                return None

        return data

    def go(self):
        self.runCommand(self._go_cmd)

    def checkEncryptionKey(self, kin):
        blen = self.keyLen()

        if len(kin) < blen:
            logging.warning('Padding key...')
            newkey = bytearray(kin)
            newkey += bytearray([0]*(blen - len(kin)))
            return newkey
        elif len(kin) > blen:
            logging.warning('Truncating key...')
            return kin[0:blen]

        return kin

    def checkPlaintext(self, text):
        blen = self.textLen()

        if len(text) < blen:
            logging.warning('Padding plaintext...')
            newtext = bytearray(text)
            newtext += bytearray([0] * (blen - len(text)))
            return newtext
        elif len(text) > blen:
            logging.warning('Truncating plaintext...')
            return text[0:blen]
        return text

    def checkMask(self, mask):
        blen = self.maskLen()

        if len(mask) < blen:
            logging.warning('Padding mask...')
            newmask = bytearray(mask)
            newmask += bytearray([0] * (blen - len(mask)))
            return newmask
        elif len(mask) > blen:
            logging.warning('Truncating mask...')
            return mask[0:blen]
        return mask

    def getExpected(self):
        """Based on key & text get expected if known, otherwise returns None"""
        if self.textLen() == 16:
            return TargetTemplate.getExpected(self)
        else:
            return None

    def write(self, data):
        """ Writes data to the target over serial.

        Args:
            data (str): Data to write over serial.

        Raises:
            Warning: Target not connected

        .. versionadded:: 5.1
            Added target.write()
        """
        if not self.connectStatus:
            raise Warning("Target not connected")

        try:
            self.ser.write(data)
        except USBError:
            self.dis()
            raise Warning("Error in target. It may have been disconnected")
        except Exception as e:
            self.dis()
            raise e

    def read(self, num_char = 0, timeout = 250):
        """ Reads data from the target over serial.

        Args:
            num_char (int, optional): Number of byte to read. If 0, read all
                data available. Defaults to 0.
            timeout (int, optional): How long in ms to wait before returning.
                If 0, block until data received. Defaults to 250.

        Returns:
            String of received data.

        .. versionadded:: 5.1
            Added target.read()
        """
        if not self.connectStatus:
            raise Warning("Target not connected")
        try:
            if num_char == 0:
                num_char = self.ser.inWaiting()
            return self.ser.read(num_char, timeout)
        except USBError:
            self.dis()
            raise Warning("Error in target. It may have been disconnected")
        except Exception as e:
            self.dis()
            raise e


    def simpleserial_wait_ack(self, timeout=500):
        """Waits for an ack from the target for timeout ms

        Args:
            timeout (int, optional): Time to wait for an ack in ms. If 0, block
                until we get an ack. Defaults to 500.


        Raises:
            Warning: Target not connected.

        .. versionadded:: 5.1
            Added target.simpleserial_wait_ack
        """

        data = self.read(4, timeout = timeout)
        if len(data) > 1:
            if data[0] != 'z':
                logging.error("Ack error: {}".format(data))
                return False
        else:
            logging.error("Target did not ack")
            return False
        return True

    def simpleserial_write(self, cmd, num, end='\n'):
        """ Writes a simpleserial command to the target over serial.

        Writes 'cmd' + ascii(num) + 'end' over serial. Flushes the read and
        write buffers before writing.

        Args:
            cmd (str): String to start the simpleserial command with. For
                'p'.
            num (bytearray): Number to write as part of command. For example,
                the 16 byte plaintext for the 'p' command. Converted to ascii
                before being sent.
            end (str, optional): String to end the simpleserial command with.
                Defaults to '\\n'.

        Example:
            Sending a 'p' command::

                key, pt = ktp.new_pair()
                target.simpleserial_write('p', pt)

        Raises:
            Warning: Write attempted while disconnected or error during write.

        .. versionadded:: 5.1
            Added target.simpleserial_write()
        """
        self.ser.flush()
        cmd += binascii.hexlify(num).decode() + end
        self.write(cmd)

    def simpleserial_read(self, cmd, pay_len, end='\n', timeout=250, ack=True):
        r""" Reads a simpleserial command from the target over serial.

        Reads a command starting with <start> with an ASCII encoded bytearray
        payload of length exp_len*2 (i.e. exp_len=16 for an AES128 key) and
        ending with <end>. Converts the payload to a bytearray. Will ignore
        non-ASCII bytes in the payload, but warn the user of them.

        Args:
            cmd (str): Expected start of the command. Will warn the user if
                the received command does not start with this string.
            pay_len (int): Expected length of the returned bytearray in number
                of bytes. Note that SimpleSerial commands send data as ASCII;
                this is the length of the data that was encoded.
            end (str, optional): Expected end of the command. Will warn the
                user if the received command does not end with this string.
                Defaults to '\n'
            timeout (int, optional): Value to use for timeouts during reads in
                ms. If 0, block until all expected data is returned. Defaults
                to 250.
            ack (bool, optional): Expect an ack at the end for SimpleSerial
                >= 1.1. Defaults to True.

        Returns:
            The received payload as a bytearray or None if the read failed.

        Example:
            Reading ciphertext back from the target after a 'p' command::

                ct = target.simpleserial_read('r', 16)

        Raises:
            Warning: Device did not ack or error during read.

        .. versionadded:: 5.1
            Added target.simpleserial_read()
        """
        cmd_len = len(cmd)
        ascii_len = pay_len * 2
        recv_len = cmd_len + ascii_len + len(end)
        response = self.read(recv_len, timeout=timeout)

        payload = bytearray(pay_len)
        if cmd_len > 0:
            if response[0:cmd_len] != cmd:
                logging.warning("Unexpected start to command: {}".format(
                    response[0:cmd_len]
                ))
                return None
        idx = cmd_len
        for i in range(0, pay_len):
            try:
                payload[i] = int(response[idx:(idx + 2)], 16)
            except ValueError as e:
                logging.warning("ValueError: {}".format(e))
            idx += 2

        if len(end) > 0:
            if response[(idx):(idx + len(end))] != end:
                logging.warning("Unexpected end to command: {}".format(
                    response[(idx):(idx+len(end))]))
                return None

        if ack:
            self.simpleserial_wait_ack(timeout)

        return payload

    def set_key(self, key, ack=True, timeout=250):
        """Checks if key is different than the last one sent. If so, send it.

        Uses simpleserial_write('k')

        Args:
            key (bytearray): key to send
            ack (bool, optional): Wait for ack after sending key. Defaults to
                True.
            timeout (int, optional): How long in ms to wait for the ack.
                Defaults to 250.

        Raises:
            Warning: Device did not ack or error during read.

        .. versionadded:: 5.1
            Added target.set_key()
        """
        if self.last_key != key:
            self.last_key = key
            self.simpleserial_write('k', key)
            if ack:
                self.simpleserial_wait_ack(timeout)

    def in_waiting(self):
        """Returns the number of characters available from the serial buffer.

        Returns:
            The number of characters available via a target.read() call.

        .. versionadded:: 5.1
            Added target.in_waiting()
        """
        return self.ser.inWaiting()

    inWaiting = camel_case_deprecated(in_waiting)

    def flush(self):
        """Removes all data from the serial buffer.

        .. versionadded:: 5.1
            Added target.flush()
        """
        self.ser.flush()






#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2015, NewAE Technology Inc
# All rights reserved.
#
# Authors: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

import pickle


class PartialReconfigData(object):
    """ Handles a single partial reconfiguration file. """
    def load(self, fname):
        self.configData = pickle.load(open(fname, 'r'))
        klist = list(self.configData['values'].keys())
        self.limits = (min(klist), max(klist))

    def getParamSpec(self, name):
        spec = {}
        spec['name'] = name
        spec['type'] = 'int'
        spec['limits'] = self.limits
        spec['reconfigData'] = self.configData

    def getPartialBitstream(self, indx):
        diffs = self.configData['values'][indx]
        data = list(self.configData['base'])

        #Update base
        for diff in diffs:
            data[diff[0]] = diff[1]
        return data


class PartialReconfigDataMulti(object):
    """
    Handles combining multiple partial reconfiguration files into a single file. Relies heavily on
    modifications not happeneing to the same location. 
    """
    def __init__(self):
        self.dataList = []
        self.limitList = []

    def load(self, filelike):
        data = pickle.load(filelike)
        self.dataList.append(data)
        klist = list(data['values'].keys())
        self.limitList.append((min(klist), max(klist)))

    def getPartialBitstream(self, indxlst):

        data = list(self.dataList[0]['base'])
        baseref = data[:]

        diffs = []
        for i, cfg in enumerate(self.dataList):
            diff = cfg['values'][indxlst[i]]
            # print diff
            diffs.append(diff)

            #Update base
            for d in diff:
                ## Check if multiple updates to same location (debug info)
                #if data[d[0]] != baseref[d[0]]:
                #    print "COLLISION & %d, %x/%x --> %x --> %x" % (d[0], baseref[d[0]], list(self.dataList[1]['base'])[d[0]], data[d[0]], d[1])
                data[d[0]] = d[1]

        return data


class PartialReconfigDataOffsetWidth(object):
    """ Handles a special file with two changes blended into one file. """

    # NOTE: This module was somewhat an emergency hack to fix the LX9 configuration failures due to PR. Thus
    #       stuff like the offset/width format is hard coded. It's not ideal but it works.

    def load(self, filelike):
        data = pickle.load(filelike)
        self.dataList = data
        klist = list(data['values'].keys())

        offset = [klist[i][0] for i in range(len(klist))]
        width = [klist[i][1] for i in range(len(klist))]

        self.limitList = []
        self.limitList.append((min(width), max(width)))
        self.limitList.append((min(offset), max(offset)))

    def getPartialBitstream(self, indxlst):
        data = list(self.dataList['base'])

        # Expected is "(offset, width)"
        diff = self.dataList['values'][(indxlst[1], indxlst[0])]

        # Update base
        for d in diff:
            data[d[0]] = d[1]

        return data


class PartialReconfigConnection(object):
    reconfig = 52
    CODE_READ       = 0x80
    CODE_WRITE = 0xC0

    def __init__(self):
        self.oa = None

    def con(self, oadc):
        self.oa = oadc

    def dis(self):
        self.oa = None

    def isPresent(self):
        resp = self.oa.sendMessage(self.CODE_READ, self.reconfig, Validate=False, maxResp=1)
        if resp is None:
            return False

        return True

    def program(self, cfgdata):

        if self.oa is None:
            return

        # Reset PR System
        self.oa.sendMessage(self.CODE_WRITE, self.reconfig, [0x20], Validate=False)
        self.oa.sendMessage(self.CODE_WRITE, self.reconfig, [0x00], Validate=False)

        # Load Data
        dataarray = [0x00]

        # print len(cfgdata)
        if len(cfgdata) > 512:
            raise IOError("PR Data too long!")

        for data in cfgdata:
            #data = int(t, 2)
            msb = data >> 8;
            lsb = data & 0xff;
            dataarray.append(msb)
            dataarray.append(lsb)


        self.oa.sendMessage(self.CODE_WRITE, self.reconfig, dataarray, Validate=False)

        # stat = self.oa.sendMessage(self.CODE_READ, self.reconfig, Validate=False, maxResp=1)
        # print "%02x" % stat[0]

        # Write Data
        self.oa.sendMessage(self.CODE_WRITE, self.reconfig, [0x1A], Validate=False)

        # Check PR Loaded OK
        stat = self.oa.sendMessage(self.CODE_READ, self.reconfig, Validate=False, maxResp=1)

        if stat[0] != 0x00:
            raise IOError("Failed to load PR Data - Status Register = 0x%02x, should be 0x00" % stat[0])
        # print "%02x" % stat[0]
